/***************************************************************************
    smb4kpasswdreader.cpp  -  Reads passwords. Used by several classes.
                             -------------------
    begin                : Mit Mai 14 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qlabel.h>

// KDE includes
#include <kapplication.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kurl.h>

// system includes
#include <cstdlib>

// application specific includes
#include "smb4kpasswdreader.h"


Smb4KPasswdReader::Smb4KPasswdReader( QWidget *parent, const char *name ) : QObject( parent, name )
{
  m_widget = parent;
  m_name = name;
  
  // First we need the directory.
  KStandardDirs *stddir = new KStandardDirs();
  QString dir = locateLocal( "appdata", QString::null, KGlobal::instance() );
  if ( !stddir->exists( dir ) )
    stddir->makeDir( dir );

  // Do not use the KConfig object, that is provided by kapp->config().
  // We want a separate one for the passwords.
  m_config = new KConfig( "passwords", false, true, "appdata" );
  // Give it strict permissions:
  m_config->setFileWriteMode( 00600 );

	// FIXME: Remove in version >= 0.4.0.
  // Remove old entries.
  if ( m_config->hasGroup( "Hosts" ) )
    m_config->deleteGroup( "Hosts" );

  // Holds the number of items in the password file.
  m_num = 0;
  // Get the number of items in the password file.
  if ( m_config->hasGroup( "Logins" ) )
  {
    m_config->setGroup( "Logins" );
    for ( int index = 0; ; index++ )
    {
      if ( !m_config->readListEntry( QString( "%1" ).arg( index ), ',' ).isEmpty() )
        m_num++;
      else
        break;
    }
  }
  
  // Get or create the key and convert the not obfuscated passwords,
  // if there are any:
  KConfig *global_config = kapp->config();
  global_config->setGroup( "Authentication" );
  
  if ( global_config->hasKey( "Random Number") )
  {
    // FIXME: Instead of 0 we can put rand() here after removal of the stuff
    // below.
    m_key = global_config->readNumEntry( "Random Number", 0 );
  }
  else if ( !global_config->hasKey( "Random Number" ) )
  {
    m_key = rand();
    
    // Convert old passwords, if necessary.
    // FIXME: Transition stuff. Remove for versions >= 0.4.0.
    if ( m_config->hasGroup( "Logins" ) )
    {
      m_config->setGroup( "Logins" );
        
      for ( int index = 0; ; index++ )
      {
        QStringList data = m_config->readListEntry( QString( "%1" ).arg( index ), ',' );
        if ( !data.isEmpty() )
        {
          const QString &temp = data[4];
          QString pw;
          uint pos( 0 );
          while ( pos < temp.length() )
          {
            pw[pos] = (char)( (int)temp[pos]+m_key%62 );
            pos++;
          }
          data[4] = pw;
          m_config->writeEntry( QString( "%1" ).arg( index ), data );
          m_config->sync();
        }
        else
          break;      
      }
    }
    
    global_config->writeEntry( "Random Number", m_key );
  }
}


Smb4KPasswdReader::~Smb4KPasswdReader()
{
}


/****************************************************************************
   Functions that read the authentication info.
****************************************************************************/


QStringList Smb4KPasswdReader::read( const QString &workgroup, const QString &host, const QString &share )
{
  // Clear the m_entries string list.
  m_entries.clear();

  if ( m_config->hasGroup( "Logins" ) )
  {
    m_config->setGroup( "Logins" );
    QStringList loginData;
    for ( int index = 0; ; index++ )
    {
      loginData = m_config->readListEntry( QString( "%1" ).arg( index ), ',' );
      
      // "Decrypt" the password.
      if ( !loginData.isEmpty() )
        loginData[4] = obfuscate( loginData[4], false );
            
      if ( loginData.isEmpty() )
      {
        m_num = index;
        break;
      }
      else if ( loginData[0] == workgroup && loginData[1] == host && ( loginData[2].lower() == share.lower() || loginData[2] == "*" ) )
      {
        // The exact match is to be favoured.
        if ( loginData[2] == "*" && m_entries.isEmpty() )
        {
					m_entries.append( loginData[3] );
          m_entries.append( loginData[4] );
        }
        else if ( loginData[2].lower() == share.lower() )
        {
          m_entries.clear();
          m_entries.append( loginData[3] );
          m_entries.append( loginData[4] );
          break;
        }
      }
      else if ( loginData[0] == workgroup && loginData[1] == host && share.isEmpty() )
      {
			  if ( loginData[2] == "*" )
				{
          m_entries.append( loginData[3] );
          m_entries.append( loginData[4] );
			  }
        break;
      }
      else
        loginData.clear();
    }
    loginData.clear();
  }

  // Make sure, the list is not empty.
  if ( m_entries.count() == 0 )
  {
    kapp->config()->setGroup( "Authentication" );
    
    if ( kapp->config()->readBoolEntry( "Default Authentication", false ) )
    {
      m_config->setGroup( "Default" );
      QString pass = obfuscate( m_config->readEntry( "Password", QString::null ), false );
      m_entries.append( m_config->readEntry( "User", QString::null ) );
      m_entries.append( pass );
    }
    else
    {
      m_entries.append( QString::null );
      m_entries.append( QString::null );
    }
  }
  
  // Return the list.
  return m_entries;
}


QStringList Smb4KPasswdReader::read( const QString &host, const QString &share )
{
  // Clear the m_entries string list.
  m_entries.clear();

  if ( m_config->hasGroup( "Logins" ) )
  {
    m_config->setGroup( "Logins" );
    QStringList loginData;
    for ( int index = 0; ; index++ )
    {
      loginData = m_config->readListEntry( QString( "%1" ).arg( index ), ',' );
      
      // "Decrypt" the password. 
      if ( !loginData.isEmpty() )
        loginData[4] = obfuscate( loginData[4], false );
      
      if ( loginData.isEmpty() )
      {
        m_num = index;
        break;
      }
      else if ( loginData[1] == host && ( loginData[2].lower() == share.lower() || loginData[2] == "*" ) )
      {
        // The exact match is to be favoured.
        if ( loginData[2] == "*" && m_entries.isEmpty() )
        {
          m_entries.append( loginData[3] );
          m_entries.append( loginData[4] );
        }
        else if ( loginData[2].lower() == share.lower() )
        {
          m_entries.clear();
          m_entries.append( loginData[3] );
          m_entries.append( loginData[4] );
          break;
        }
      }
      else
        loginData.clear();
    }
    loginData.clear();
  }

  // Make sure, the list is not empty.
  if ( m_entries.count() == 0 )
  {
    kapp->config()->setGroup( "Authentication" );
    
    if ( kapp->config()->readBoolEntry( "Default Authentication", false ) )
    {
      m_config->setGroup( "Default" );
      QString pass = obfuscate( m_config->readEntry( "Password", QString::null ), false );
      m_entries.append( m_config->readEntry( "User", QString::null ) );
      m_entries.append( pass );    
    }
    else
    {
      m_entries.append( QString::null );
      m_entries.append( QString::null );
    }
  }
    
  return m_entries;
}


QStringList Smb4KPasswdReader::read()
{
  // Clear the entry list.
  m_entries.clear();

  if ( m_config->hasGroup( "Logins" ) )
  {
    QStringList loginData;
    m_config->setGroup( "Logins" );
    for ( int pos = 0; ; pos++ )
    {
      loginData = m_config->readListEntry( QString( "%1" ).arg( pos, 0, 10 ).stripWhiteSpace(), ',' );
      
      // "Decrypt" the password.
      if ( !loginData.isEmpty() )
      {
        loginData[4] = obfuscate( loginData[4], false );
        QString entry = loginData.join( ":" );
        m_entries.append( entry );
        loginData.clear();
      }
      else
        break;
    }
  }
  return m_entries;
}


/****************************************************************************
   Reads the default auth data.
****************************************************************************/

QStringList Smb4KPasswdReader::readDefaultAuthData()
{
  m_config->setGroup( "Default" );
  QStringList authData;
  
  authData.append( m_config->readEntry( "User", QString::null ) );
  authData.append( obfuscate( m_config->readEntry( "Password", QString::null ), false ) );

  return authData;  
}


/****************************************************************************
   Functions that write the authentication info.
****************************************************************************/


void Smb4KPasswdReader::write( const QStringList &data )
{
  m_config->setGroup( "Logins" );
  bool write = true;
  
  // Write the authentication info to Smb4K's password file.
  for ( int index = 0; ; index++ )
  {
    QStringList list = m_config->readListEntry( QString( "%1" ).arg( index, 0, 10 ).stripWhiteSpace(), ',' );
    if ( !list.isEmpty() )
    {
      if ( list[0] == data[0] && list[1] == data[1] && list[2] == data[2] && list[3] == data[3] && obfuscate( list[4], false ) == data[4] )
      {
         write = false;
         break;
      }
      else if ( list[0] == data[0] && list[1] == data[1] && list[2] == data[2] && ( list[3] != data[3] || obfuscate( list[4], false ) != data[4] ) )
      {
        QStringList data_to_write = data;
        data_to_write[4] = obfuscate( data_to_write[4], true );
        m_config->writeEntry( QString( "%1" ).arg( index, 0, 10 ), data_to_write );
        write = false;
        break;
      }
      else
        continue;
    }
    else
      break;
  }
  if ( write )
  {
    QStringList data_to_write = data;
    data_to_write[4] = obfuscate( data_to_write[4], true );
    m_config->writeEntry( QString( "%1" ).arg( m_num, 0, 10 ).stripWhiteSpace(), data_to_write );
    m_num++;
  }
  m_config->sync();
}


/****************************************************************************
   Writes the auth data that is passed by the configuration dialog on
   exit.
****************************************************************************/


void Smb4KPasswdReader::writeAuthData( const QStringList &list )
{
  // Clear all entries.
  if ( m_config->hasGroup( "Logins" ) )
  {
    m_config->deleteGroup( "Logins", true, false );
    m_config->sync();
    m_num = 0;
  }

  uint index( 0 );
  while ( index < list.count() )
  {
    QStringList data = QStringList::split( ":", list[index], false );
    write( data );
    index++;
  }
}


/****************************************************************************
   Writes the default auth data that is passed by the configuration dialog
   on exit.
****************************************************************************/


void Smb4KPasswdReader::writeDefaultAuthData( const QStringList &list )
{
  // Obfuscate the password:
  QString pass = obfuscate( list[1], true );
  
  // Now write the data:
  m_config->setGroup( "Default" );
  m_config->writeEntry( "User", list[0] );
  m_config->writeEntry( "Password", pass );
  
  m_config->sync();
}


/****************************************************************************
   This function shows the askpass dialog.
****************************************************************************/


bool Smb4KPasswdReader::askpass( const QString &workgroup, const QString &host, const QString &share )
{
  passwdDlg = new KDialogBase( KDialogBase::Plain, i18n( "Authentication" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, m_widget, m_name, false, true );
  
  // Set up the ask pass dialog.
  QFrame *frame = passwdDlg->plainPage();
  QGridLayout *layout = new QGridLayout( frame );
  layout->setSpacing( 5 );

  QLabel *pic = new QLabel( frame );
  pic->setPixmap( DesktopIcon( "personal" ) );
  pic->setMargin( 10 );

  QWidget *shareData = new QWidget( frame );
  QGridLayout *grid = new QGridLayout( shareData );
  grid->setSpacing( 5 );
  grid->setMargin( 0 );

  QLabel *label1 = new QLabel( i18n( "Workgroup:" ), shareData );
  m_workgroupLabel = new QLabel( shareData );
  QLabel *label2 = new QLabel( i18n( "Host:" ), shareData );
  m_hostLabel = new QLabel( shareData );
  QLabel *label3 = new QLabel( i18n( "Share:" ), shareData );
  m_shareLabel = new QLabel( shareData );

  grid->addWidget( label1, 1, 0, 0 );
  grid->addWidget( m_workgroupLabel, 1, 1, 0 );
  grid->addWidget( label2, 2, 0, 0 );
  grid->addWidget( m_hostLabel, 2, 1, 0 );
  grid->addWidget( label3, 3, 0, 0 );
  grid->addWidget( m_shareLabel, 3, 1, 0 );

  QLabel *user = new QLabel( i18n( "User:" ), frame );
  m_username = new KLineEdit( frame );
  QLabel *pass = new QLabel( i18n( "Password:" ), frame );
  m_password = new KLineEdit( frame );
  m_password->setEchoMode( KLineEdit::Password );

  m_use = new QCheckBox( i18n( "Use authentication for all shares." ), frame );

  QSpacerItem *spacer1 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred );

  layout->addWidget( pic, 0, 0, 0 );
  layout->addMultiCellWidget( shareData, 0, 0, 1, 3, 0 );
  layout->addWidget( user, 1, 0, 0 );
  layout->addMultiCellWidget( m_username, 1, 1, 1, 4, 0 );
  layout->addWidget( pass, 2, 0, 0 );
  layout->addMultiCellWidget( m_password, 2, 2, 1, 4, 0 );
  layout->addMultiCellWidget( m_use, 3, 3, 0, 3, 0 );
  layout->addItem( spacer1, 0, 4 );

  QStringList login = read( workgroup, host, share );

  m_workgroupLabel->setText( workgroup );
  m_hostLabel->setText( host );
  
  if ( !login[0].isEmpty() )
    m_username->setText( login[0] );

  if ( !login[1].isEmpty() )
    m_password->setText( login[1] );

  if ( share.isEmpty() )
  {
    m_use->setEnabled( false );
    m_use->setChecked( true );
    m_shareLabel->setText( "---" );
  }
  else
  {
    m_use->setEnabled( true );
    m_shareLabel->setText( share );
  }
    
  m_username->setFocus();
    
  passwdDlg->setFixedSize( passwdDlg->sizeHint() );

  QStringList auth;
  auth.append( workgroup );
  auth.append( host );
  if ( share.isEmpty() || m_use->isChecked() )
    auth.append( "*" );
  else
    auth.append( share );
  
  bool new_passwd = false;
  
  // Show it.
  if ( passwdDlg->exec() == KDialogBase::Accepted )
  {
	  // Since we want to allow the user to provide an empty password,
		// only the user name is checked to be not empty.
    if ( !m_username->text().stripWhiteSpace().isEmpty() )
    {
      auth.append( m_username->text().stripWhiteSpace() );
      auth.append( m_password->text().stripWhiteSpace() );
      write( auth );     
      new_passwd = true;
    }
  }
  
  return new_passwd;
}


/****************************************************************************
   Obfuscation of the passwords.
****************************************************************************/


const QString Smb4KPasswdReader::obfuscate( const QString &password, bool encrypt )
{
  QString passwd;
  
  if ( encrypt )
  {
    uint index( 0 );
    while ( index < password.length() )
    {
      passwd[index] = (char)( (int)password[index]+m_key%62 );
      index++;
    }
  }
  else if ( !encrypt )
  {
    uint index( 0 );
    while ( index < password.length() )
    {
      passwd[index] = (char)( (int)password[index]-m_key%62 );
      index++;
    }
  }
    
  return passwd;
}


#include "smb4kpasswdreader.moc"
