/***************************************************************************
    smb4kpasswdreader.h  -  Reads passwords. Used by several classes.
                             -------------------
    begin                : Mit Mai 14 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4KPASSWDREADER_H
#define SMB4KPASSWDREADER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qobject.h>
#include <qlabel.h>
#include <qstringlist.h>
#include <qcheckbox.h>

// KDE includes
#include <kconfig.h>
#include <kdialogbase.h>
#include <klineedit.h>


/** This is a core class. It handles the reading and writing of the
    login information. It also provides the askpass dialog. */

class Smb4KPasswdReader : public QObject
{
  Q_OBJECT

  public:
    /** The constructor. */
    Smb4KPasswdReader( QWidget *parent = 0, const char *name = 0 );
    /** The destructor. */
    ~Smb4KPasswdReader();
    /** Reads the authentication data for a share and returns it. The first entry in the
        list is the username, the second the password.
        This function takes the workgroup, the host and the share name as arguments. */
    QStringList read( const QString &workgroup, const QString &host, const QString &share );
    /** This function behaves like the one above except that it only takes the host
        name as argument. It should only be used by the Smb4KStarter class. */
    QStringList read( const QString &host, const QString &share );
    /** Reads the hole list from the config file and returns it.*/
    QStringList read();
    /** Reads the default auth data. */
    QStringList readDefaultAuthData();
    /** Write the auth data passed by the configuration dialog. */
    void writeAuthData( const QStringList &data );
    /** Writes the default authentication data to the password file. */
    void writeDefaultAuthData( const QStringList &list );
    /** This function writes passwords to the password file.
        It takes the position pos, a stringlist with the data, that
        has to be written and the type defined by Smb4KPasswdReader::Type. */
    void write( const QStringList &data );
    /** This enumeration holds the group names, that are or were used in the password file. */
    enum Groups{ Hosts, Logins };
    /** The askpass. Returns true, if the user supplied auth data and pressed OK. Otherwise
        false is returned. */
    bool askpass( const QString &workgroup, const QString &host, const QString &share );  

  private:
    /** This function obfuscates/decrypts the passwords. */
    const QString obfuscate( const QString &password, bool encrypt );
    QLabel *m_workgroupLabel;
    QLabel *m_hostLabel;
    QLabel *m_shareLabel;
    QStringList m_entries;
    KConfig *m_config;
    int m_num;
    KLineEdit *m_username;
    KLineEdit *m_password;
    QCheckBox *m_use;
//    QString m_workgroup;
//    QString m_host;
//    QString m_share;
    /** Holds the widget, the password reader is a child of. */
    QWidget *m_widget;
    /** Holds the name of the widget, the password reader is a child of. */
    const char *m_name;
    /** The key variable. */
    int m_key;
    /** The askpass widget. */
    KDialogBase *passwdDlg;
};
#endif
