/***************************************************************************
    smb4kstarter.h  -  Stuff that's carried out on start-up.
                             -------------------
    begin                : Mit Apr 30 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/*************************************************************************** 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4KSTARTER_H
#define SMB4KSTARTER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qobject.h>
#include <qstringlist.h>
#include <qvbox.h>

// KDE includes
#include <kprocess.h>
#include <kconfig.h>
#include <ksqueezedtextlabel.h>
#include <kprogress.h>

/** This is a core class. It is invoked before Smb4KApp is initialized and
    takes care of all the things, that have to be done before the main window
    is shown. */

class Smb4KStarter : public QObject
{
  Q_OBJECT
  public:
    /** The constructor. */
    Smb4KStarter( QObject *parent = 0, const char *name = 0 );
    /** The destructor. */
    ~Smb4KStarter();

    /** Does everything, that has to be done on start-up. */
    void init();
    
  protected:
    /** Starts the main application (Smb4KApp). */
    void runProgram();    

  protected slots:
    /** Closed the splash screen. */
    void slotClose();
    /** Ends the process, if shares had to be mounted. */
    void slotProcessExited( KProcess * );

  private:
    /** Build the splash screen and show it. */
    void splashScreen();
    /** Determines the operting system. */
    void detectOS();
    /** Determines the WINS server support. */
    void checkForWINS();
    /** Checks for already mounted shares and returns a list of them. */
    const QStringList checkForShares();
    /** Gets the recently used shares out of the config file and stores
        them in the private list Smb4KStarter::m_list. */
    const QStringList getSharesToMount();
    /** Mounts the recently used shares. */
    void mount();
    /** The KProcess object. */
    KProcess *m_proc;
    /** The KConfig object. */
    KConfig *m_config;
    /** This string list holds the shares, that are to be mounted. */
    QStringList m_list;
    /** This stringlist holds the already mounted SMB shares. */
    QStringList m_mounted;
    /** The path variable. */
    QString m_path;
    /** The splash screen widget. */
    QFrame *m_splashScreen;
    /** The text label of the splash screen. */
    KSqueezedTextLabel *m_infoLabel;
    /** The progress bar of the splash screen. */
    KProgress *m_progress;
    /** Holds the users UID */
    int m_uid;
    /** Holds the users GID */
    int m_gid;
};

#endif
