/***************************************************************************
    smb4k.cpp  -  The main class of Smb4K.
                             -------------------
    begin                : Sam Mr  1 14:57:21 CET 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qtooltip.h>
#include <qtimer.h>

// KDE includes
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kstdaction.h>
#include <ksystemtray.h>

// application specific includes
#include "smb4k.h"
#include "widgets/smb4kbrowserwidget.h"
#include "widgets/smb4knetworktab.h"
#include "widgets/smb4ksearchtab.h"
#include "widgets/smb4ksharewidget.h"
#include "widgets/options/smb4koptionsdlg.h"

Smb4KApp *Smb4KApp::sApp = 0L;

#define ID_STATUS_MSG 1

Smb4KApp::Smb4KApp( QWidget* , const char* name ) : KMainWindow( 0, name )
{
  sApp = this;
  
  m_config = kapp->config();

  // The caption.
  setPlainCaption( i18n( "Smb4K - The SMB Share Browser" ) );
      
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initCore();
  initView();
  initActions();

  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  abort->setEnabled( false );
  networkPreview->setEnabled( false );
  sharesMount->setEnabled( false );
  sharesUnmount->setEnabled( false );
  sharesUnmountAll->setEnabled( false );
  sharesStartFilemanager->setEnabled( false );
  networkAskPass->setEnabled( false );
}


Smb4KApp::~Smb4KApp()
{
}


void Smb4KApp::initActions()
{
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
	viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
	abort = new KAction( i18n( "&Abort" ) , "stop", i18n( "CTRL+A" ), this, SLOT( slotAbort() ), actionCollection(), "abort_action" );
	networkRescan = new KAction( i18n( "&Rescan" ), "reload", i18n( "CTRL+R" ), m_scanner, SLOT( slotInit() ), actionCollection(), "network_rescan" );
	networkSearch = new KAction( i18n( "&Search" ), "find", i18n( "CTRL+S" ), m_view, SLOT( slotShowSearchTab() ), actionCollection(), "network_search" );
	networkPreview = new KAction( i18n( "Pre&view" ), "view_icon", i18n( "CTRL+V" ), m_view->m_NetworkView, SLOT( slotPreview() ), actionCollection(), "network_preview" );
	networkAskPass = new KAction( i18n( "Au&thentication" ), "personal", i18n( "CTRL+T" ), m_view->m_NetworkView, SLOT( slotAskPass() ), actionCollection(), "network_askpass" );
	sharesMount = new KAction( i18n( "&Mount" ), "hdd_mount", i18n( "CTRL+M" ), m_view->m_NetworkView, SLOT( slotMountShare() ), actionCollection(), "shares_mount" );
	sharesUnmount = new KAction( i18n( "&Unmount" ), "hdd_unmount", i18n( "CTRL+U" ), m_view->m_ShareView, SLOT( slotUnmountShare() ), actionCollection(), "shares_unmount_share" );
	sharesUnmountAll = new KAction( i18n( "U&nmount All" ), "gear", i18n( "CTRL+N" ), m_view->m_ShareView, SLOT( slotUnmountAllShares() ), actionCollection(), "shares_unmount_all" );
	sharesStartFilemanager = new KAction( i18n( "&Konqueror" ), "kfm_home", i18n( "CTRL+K" ), m_view->m_ShareView, SLOT( slotOpenFilemanager() ), actionCollection(), "shares_start_filemanager" );
	settingSmb4KOptions = KStdAction::preferences( this, SLOT( slotSmb4KOptionsDlg() ), actionCollection() );    fileQuit->setStatusText( i18n( "Quits the application" ) );
	viewToolBar->setStatusText( i18n( "Enables/disables the toolbar" ) );  viewStatusBar->setStatusText( i18n( "Enables/disables the statusbar" ) );

	abort->setStatusText( i18n( "Abort" ) );
  networkRescan->setStatusText( i18n( "Rescan network" ) );
  networkSearch->setStatusText( i18n( "Network search tab" ) );
  networkPreview->setStatusText( i18n( "Preview dialog" ) );
  networkAskPass->setStatusText( i18n( "Authentication dialog" ) );
  sharesMount->setStatusText( i18n( "Mount share" ) );
  sharesUnmount->setStatusText( i18n( "Unmount share" ) );
  sharesUnmountAll->setStatusText( i18n( "Unmount all shares" ) );
  sharesStartFilemanager->setStatusText( i18n( "Open share with Konqueror" ) );
  settingSmb4KOptions->setStatusText( i18n( "Options" ) );

	// use the absolute path to your smb4kui.rc file for testing purpose in createGUI();
	createGUI();}


void Smb4KApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
	// STATUSBAR
	// TODO: add your own items you need for displaying current application status.
	statusBar()->insertItem( i18n( "Ready." ), ID_STATUS_MSG, 1, false );
	statusBar()->setItemAlignment( ID_STATUS_MSG, AlignLeft );

	// Labels:
	m_winsLabel = new QLabel( statusBar() );
	m_winsLabel->setAlignment( Qt::AlignCenter );
	m_listLabel = new QLabel( statusBar() );
  m_listLabel->setAlignment( Qt::AlignCenter );
  QLabel *version = new QLabel( QString( "Smb4K %1" ).arg( VERSION ), statusBar() );
	version->setAlignment( Qt::AlignCenter );
  version->setFixedWidth( version->sizeHint().width() + 5 );

  m_progress = new KProgress( statusBar(), 0, 0 );
  m_progress->setFixedWidth( 75 );
  m_progress->setFixedHeight( version->fontMetrics().height() );
  m_progress->setPercentageVisible( false );

  m_progTimer = new QTimer( this );
  connect( m_progTimer, SIGNAL( timeout() ), SLOT( slotSetProgress() ) );

  statusBar()->addWidget( m_winsLabel, false );
  statusBar()->addWidget( m_listLabel, false );
  statusBar()->addWidget( m_progress, false );
  statusBar()->addWidget( version, false );

  QToolTip::add( m_winsLabel, i18n( "WINS server" ) );
  QToolTip::add( m_listLabel, i18n( "Browse list retrieval" ) );
  QToolTip::add( version, i18n( "Version" ) );
}


void Smb4KApp::initCore()
{
  // The mounter.
  m_mounter = new Smb4KMounter( this );
  // The scanner.
  m_scanner = new Smb4KScanner( this );
  // The password reader.
  m_reader = new Smb4KPasswdReader( this );

  // Connections
  connect( m_scanner, SIGNAL( running( bool ) ), this, SLOT( slotScannerRunning( bool ) ) );
	connect( m_mounter, SIGNAL( running( bool, int ) ), this, SLOT( slotMounterRunning( bool, int ) ) );
}


void Smb4KApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
	// connect the widget to your document to display document contents.
  m_view = new Smb4KView( this );
  setCentralWidget(m_view);
}


void Smb4KApp::saveOptions()
{
  m_config->setGroup("General Options");
  m_config->writeEntry( "Main Window Geometry" , size());
  m_config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  m_config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  m_config->writeEntry("Toolbar Position", (int)toolBar("mainToolBar")->barPos());
	m_config->writeEntry("Toolbar Text Position", toolBar("mainToolBar")->iconText());
	m_config->writeEntry("Toolbar Icon Size", toolBar("mainToolBar")->iconSize());
	// Put the sizes of the QSplitters into the config file.
	m_config->writeEntry( "Vertical Splitter Sizes", m_view->sizes() );
	m_config->writeEntry( "Horizontal Splitter Sizes", m_view->m_container->sizes() );}


void Smb4KApp::readOptions()
{
	
  m_config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = m_config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = m_config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("Toolbar Position", KToolBar::Top);
	toolBar("mainToolBar")->setBarPos(toolBarPos);

  KToolBar::IconText toolBarIconText;
  toolBarIconText=(KToolBar::IconText) m_config->readNumEntry("Toolbar Text Position", KToolBar::IconOnly);
	toolBar("mainToolBar")->setIconText(toolBarIconText);
  toolBar("mainToolBar")->setIconSize( m_config->readNumEntry( "Toolbar Icon Size", 0 ) );

  // FIXME: Transition stuff: Renamed KConfig entry. Remove this 'if' stuff in version 0.4.0.
	QSize size;
  if ( m_config->hasKey( "Geometry" ) )
  {
    size = m_config->readSizeEntry( "Geometry" );
    m_config->deleteEntry( "Geometry" );
  }
  else
    size = m_config->readSizeEntry( "Main Window Geometry" );

  if(!size.isEmpty())
  {
    resize(size);
  }

  // Read the sizes of the QSplitters.
  // FIXME: Transition stuff: Renamed KConfig entry. Remove part of this 'if' stuff in version 0.4.0.
	if ( m_config->hasKey( "Splitter1" ) )
  {
    m_view->setSizes( m_config->readIntListEntry( "Splitter1" ) );
    m_config->deleteEntry( "Splitter1" );
  }
  else if ( m_config->hasKey( "Vertical Splitter Sizes" ) )
    m_view->setSizes( m_config->readIntListEntry( "Vertical Splitter Sizes" ) );

  if ( m_config->hasKey( "Splitter2" ) )
  {
    m_view->m_container->setSizes( m_config->readIntListEntry( "Splitter2" ) );
    m_config->deleteEntry( "Splitter2" );
  }
  else if ( m_config->hasKey( "Horizontal Splitter Sizes" ) )
    m_view->m_container->setSizes( m_config->readIntListEntry( "Horizontal Splitter Sizes" ) );

	// Set-up the status bar.
  m_config->setGroup( "Browse Options" );
  if ( m_config->readEntry( "Browse List", "nmblookup" ) == "nmblookup" )
    m_listLabel->setText( i18n( "Network" ) );
  else
    m_listLabel->setText( m_config->readEntry( "Query Host" ) );

	if ( m_config->readBoolEntry( "WINS Support", false ) )
    m_winsLabel->setText( m_config->readEntry( "WINS Server" ) );
  else
    m_winsLabel->setText( i18n( "None" ) );

  m_listLabel->setFixedWidth( m_listLabel->sizeHint().width() + 5 );
  m_winsLabel->setFixedWidth( m_winsLabel->sizeHint().width() + 5 );

  // EXTERNAL configuration options:
  m_scanner->readOptions();
  m_mounter->readOptions();
  m_view->m_NetworkView->readOptions();
  m_view->m_ShareView->readOptions();
}


bool Smb4KApp::queryExit()
{
  saveOptions();
  return true;
}


bool Smb4KApp::queryClose()
{
  m_view->m_ShareView->exit();
  m_mounter->exit();
  return true;
}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void Smb4KApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  close();
}


void Smb4KApp::slotAbort()
{
  slotStatusMsg( i18n( "Aborting..." ) );
  m_scanner->abort();
  m_mounter->abort();
  slotStatusMsg( i18n( "Done." ) );
}


void Smb4KApp::slotScannerRunning( bool on )
{
  if ( on )
  {
    abort->setEnabled( true );
    networkRescan->setEnabled( false );
    slotStatusMsg( i18n( "Searching network..." ) );

    // Start the moving progress bar:
    m_progTimer->stop();
    m_progress->setTotalSteps( 100 );
    m_progress->reset();
    m_progress->setTotalSteps( 0 );
    m_progTimer->start( 25, false );
  }
  else if ( !on )
  {
    // Kill the moving progress bar:
    m_progTimer->stop();
    m_progress->setTotalSteps( 100 );
    m_progress->reset();

    abort->setEnabled( false );
    networkRescan->setEnabled( true );
    slotStatusMsg( i18n( "Done." ) );
  }
}


void Smb4KApp::slotMounterRunning( bool on, int todo )
{
  if ( on )
  {
    abort->setEnabled( true );
    if ( todo == Smb4KMounter::Mount )
      slotStatusMsg( i18n( "Mounting share..." ) );
    else if ( todo == Smb4KMounter::Unmount )
      slotStatusMsg( i18n( "Unmounting share..." ) );
    else if ( todo == Smb4KMounter::UnmountAll )
      slotStatusMsg( i18n( "Unmounting all shares..." ) );

    // Start the moving progress bar:
    m_progTimer->stop();
    m_progress->setTotalSteps( 100 );
    m_progress->reset();
    m_progress->setTotalSteps( 0 );
    m_progTimer->start( 25, false );
  }
  else if ( !on )
  {
    // Kill the moving progress bar:
    m_progTimer->stop();
    m_progress->setTotalSteps( 100 );
    m_progress->reset();

    slotStatusMsg( i18n( "Done." ) );
    abort->setEnabled( false );
  }
}


void Smb4KApp::slotSmb4KOptionsDlg()
{
  Smb4KOptionsDlg *configDlg = new Smb4KOptionsDlg( this );
  configDlg->show();
}


void Smb4KApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void Smb4KApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void Smb4KApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void Smb4KApp::slotSetProgress()
{
  m_progress->setProgress( m_progress->progress()+1 );
}


void Smb4KApp::slotReadOptions()
{
  readOptions();
}


#include "smb4k.moc"
