/***************************************************************************
    smb4k.h  -  The main class of Smb4K.
                             -------------------
    begin                : Sam Mr  1 14:57:21 CET 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4K_H
#define SMB4K_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qlabel.h>

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kconfig.h>
#include <klistview.h>
#include <kiconview.h>
#include <kprogress.h>

// application specific includes
#include "smb4kview.h"
#include "core/smb4kscanner.h"
#include "core/smb4kmounter.h"
#include "core/smb4kpasswdreader.h"

#define sapp Smb4KApp::smb4kapp()

/**
  * The base class for Smb4K application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of Smb4KView creates your center view, which is connected
	* to the window's Doc object.
  * Smb4KApp reimplements the methods that KMainWindow provides for main window handling and supports
	* full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
	* @version KDevelop version 1.2 code generation
  */
class Smb4KApp : public KMainWindow
{
  Q_OBJECT

  friend class Smb4KView;
  friend class Smb4KBrowserWidget;
  friend class Smb4KShareWidget;

  public:
    /** construtor of Smb4KApp, calls all init functions to create the application.     */
    Smb4KApp( QWidget* parent=0, const char *name=0 );
    /** The destructor. */
    ~Smb4KApp();

    static Smb4KApp *smb4kapp() { return sApp; }

    /** Returns the password reader. */
    Smb4KPasswdReader *passwdReader() { return m_reader; }
    /** Returns the mounter. */
    Smb4KMounter *mounter() { return m_mounter; }
    /** Returns the scanner. */
    Smb4KScanner *scanner() { return m_scanner; }
    /** Returns the pointer to the Smb4KView class. */
    Smb4KView *view() { return m_view; }

    /** read general Options again and initialize all variables like the recent file list     */
    void readOptions();


  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
		 * file
     */
    void saveOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view */
    void initView();
    /** Initializes the core applications. */
    void initCore();

    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
		* Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's
		* properties.
		* @see KTMainWindow#queryExit
		* @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** Do the last tasks before exiting. */
    virtual bool queryClose();

  public slots:
    void slotFileQuit();
    void slotViewToolBar();
    void slotViewStatusBar();
    void slotStatusMsg(const QString &text);

  protected slots:
    /** Aborts any action of the program */
    void slotAbort();
    /** This slot is called, if the scanner is running. */
    void slotScannerRunning( bool on );
    /** This slot is called, if the mounter is running. */
    void slotMounterRunning( bool on, int todo );
    /** Opens the configuration dialog. */
    void slotSmb4KOptionsDlg();
    /** Sets the progress bar. */
    void slotSetProgress();
    /** This slot is invoked, if the options have to be reread. */
    void slotReadOptions();

  private:
    static Smb4KApp *sApp;
    /** the configuration object of the application */
    KConfig *m_config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    Smb4KView *m_view;
    /** The scanner. */
    Smb4KScanner *m_scanner;
    /** The mounter. */
    Smb4KMounter *m_mounter;
    /** The password reader. */
    Smb4KPasswdReader *m_reader;

    /** Label to hold the WINS server info in the status bar. */
    QLabel *m_winsLabel;
    /** Label to hold the browse list retrieval info in the status bar. */
    QLabel *m_listLabel;
    /** The progress bar for the status bar. */
    KProgress *m_progress;
    /** The timer, that drives the progress bar. */
    QTimer *m_progTimer;

    KAction* fileQuit;
    KToggleAction* viewToolBar;
    KToggleAction* viewStatusBar;
    KAction *networkRescan;
    KAction *abort;
    KAction *networkSearch;
    KAction *sharesMount;
    KAction *sharesUnmount;
    KAction *sharesUnmountAll;
    KAction *sharesStartFilemanager;
    KAction *settingSmb4KOptions;
    KAction *networkPreview;
    KAction *networkAskPass;
};
 
#endif // SMB4K_H
