/***************************************************************************
    smb4kauthoptions  -  description
                             -------------------
    begin                : Sa Nov 15 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qvbox.h>

// KDE includes
#include <kmessagebox.h>
#include <kiconloader.h>
#include <klocale.h>

// application specific includes
#include "smb4kauthoptions.h"

Smb4KAuthOptions::Smb4KAuthOptions( QWidget *parent, const char *name ) : QWidget( parent, name )
{
  QGridLayout *grid = new QGridLayout( this );
  grid->setSpacing( 5 );
  
  //
  // Default Authentication
  //
  QVGroupBox *box1 = new QVGroupBox( i18n( "Default Login" ), this );
  box1->setInsideMargin( 10 );
  
  QLabel *description1 = new QLabel( i18n( "This login will be used, if none has been defined for a share." ), box1 );
  description1->setTextFormat( Qt::RichText );
  
  m_defaultAuth = new QCheckBox( i18n( "Use default login." ), box1 );
  
  m_authWidget = new QWidget( box1 );
  QGridLayout *box1Grid = new QGridLayout( m_authWidget );
  box1Grid->setSpacing( 10 );
  
  QSpacerItem *spacer1 = new QSpacerItem( 25, 0, QSizePolicy::Fixed, QSizePolicy::Fixed );
  QLabel *login = new QLabel( i18n( "User:" ), m_authWidget );
  m_defaultLogin = new KLineEdit( QString::null, m_authWidget );
  m_defaultLogin->setMinimumWidth( 150 );
  QLabel *passwd = new QLabel( i18n( "Password:" ), m_authWidget );
  m_defaultPasswd = new KLineEdit( QString::null, m_authWidget );
  m_defaultPasswd->setMinimumWidth( 150 );
  m_defaultPasswd->setEchoMode( KLineEdit::Password );
  
  box1Grid->addMultiCell( spacer1, 0, 1, 0, 0, 0 );
  box1Grid->addWidget( login, 0, 1 );
  box1Grid->addWidget( m_defaultLogin, 0, 2 );
  box1Grid->addWidget( passwd, 1, 1 );
  box1Grid->addWidget( m_defaultPasswd, 1, 2 );
  
  //
  // Authentication for single shares.
  //
  QVGroupBox *box2 = new QVGroupBox( i18n( "Hosts and Shares" ), this );
  box2->setInsideMargin( 10 );
  
  QLabel *description2 = new QLabel( i18n( "Here you should enter password protected hosts and shares, that you want to log in." ), box2 );
  description2->setTextFormat( Qt::RichText );

  QWidget *w2 = new QWidget( box2 );
  QGridLayout *w2Grid = new QGridLayout( w2 );
  w2Grid->setSpacing( 10 );
  
  // The line edits:
  QLabel *a = new QLabel( i18n( "Workgroup:" ), w2 );
  m_workgroupEdit = new KLineEdit( QString::null, w2 );
  m_workgroupEdit->setMinimumWidth( 150 );
  
  QLabel *b = new QLabel( i18n( "Host:" ), w2 );
  m_hostEdit = new KLineEdit( QString::null, w2 );
  m_hostEdit->setMinimumWidth( 150 );
  
  m_oneShare = new QCheckBox( i18n( "Share:" ), w2 );
  m_shareEdit = new KLineEdit( QString::null, w2 );
  m_shareEdit->setEnabled( false );
  m_shareEdit->setMinimumWidth( 150 );
  
  QLabel *c = new QLabel( i18n( "User:" ), w2 );
  m_loginEdit = new KLineEdit( QString::null, w2 );
  m_loginEdit->setMinimumWidth( 150 );
  
  QLabel *d = new QLabel( i18n( "Password:" ), w2 );
  m_passwdEdit = new KLineEdit( QString::null, w2 );
  m_passwdEdit->setEchoMode( KLineEdit::Password );
  m_passwdEdit->setMinimumWidth( 150 );
  
  // Set the tab order.
  setTabOrder( m_workgroupEdit, m_hostEdit );
  setTabOrder( m_hostEdit, m_oneShare );
  setTabOrder( m_oneShare, m_shareEdit );
  setTabOrder( m_shareEdit, m_loginEdit );
  setTabOrder( m_loginEdit, m_passwdEdit );
  
  QWidget *buttons1 = new QWidget( w2 );
  QGridLayout *buttons1Layout = new QGridLayout( buttons1 );
  buttons1Layout->setSpacing( 10 );
  QSpacerItem *spacer2 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred );
  m_add = new KPushButton( i18n( "A&dd" ), buttons1 );
  m_clear = new KPushButton( i18n( "C&lear" ), buttons1 );
  buttons1Layout->addItem( spacer2, 0, 0 );
  buttons1Layout->addWidget( m_add, 0, 1 );
  buttons1Layout->addWidget( m_clear, 0, 2 );
  
  w2Grid->addWidget( a, 0, 0 );
  w2Grid->addWidget( m_workgroupEdit, 0, 1 );  
  w2Grid->addWidget( b, 1, 0 );
  w2Grid->addWidget( m_hostEdit, 1, 1 );
  w2Grid->addWidget( m_oneShare, 2, 0 );
  w2Grid->addWidget( m_shareEdit, 2, 1 );
  w2Grid->addWidget( c, 0, 2 );
  w2Grid->addWidget( m_loginEdit, 0, 3 );
  w2Grid->addWidget( d, 1, 2 );
  w2Grid->addWidget( m_passwdEdit, 1, 3 );
  w2Grid->addMultiCellWidget( buttons1, 2, 2, 2, 3 );

  m_loginData = new KListView( box2 );
  m_loginData->addColumn( i18n( "Workgroup" ), -1 );
  m_loginData->addColumn( i18n( "Host" ), -1 );
  m_loginData->addColumn( i18n( "Share" ), -1 );
  m_loginData->addColumn( i18n( "User" ), -1 );
  m_loginData->addColumn( QString::null, 0 );
  m_loginData->setAllColumnsShowFocus( true );
               
  // Fill the main grid.
  grid->addMultiCellWidget( box1, 0, 0, 0, 1, 0 );
  grid->addMultiCellWidget( box2, 0, 0, 2, 4, 0 );
    
  m_edit = false;

  connect( m_add, SIGNAL( clicked() ), SLOT( slotAddClicked() ) );
  connect( m_clear, SIGNAL( clicked() ), SLOT( slotClearClicked() ) );
  connect( m_loginData, SIGNAL( rightButtonClicked( QListViewItem *, const QPoint &, int ) ), SLOT( slotContextMenu( QListViewItem *, const QPoint &, int ) ) );
  connect( m_oneShare, SIGNAL( stateChanged( int ) ), SLOT( slotShareStateChanged( int ) ) );
  connect( m_defaultAuth, SIGNAL( stateChanged( int ) ), SLOT( slotAuthDataStateChanged( int ) ) );
}


Smb4KAuthOptions::~Smb4KAuthOptions()
{
}


void Smb4KAuthOptions::clear()
{
  m_workgroupEdit->clear();
  m_hostEdit->clear();
  m_loginEdit->clear();
  m_passwdEdit->clear();
  m_shareEdit->clear();
  m_shareEdit->setEnabled( false );
  m_oneShare->setChecked( false );
}


/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////


void Smb4KAuthOptions::slotAddClicked()
{
  if ( m_edit )
	{
    if ( !m_workgroupEdit->text().isEmpty() )
      m_loginData->currentItem()->setText( 0, m_workgroupEdit->text() );

		if ( !m_hostEdit->text().isEmpty() )
      m_loginData->currentItem()->setText( 1, m_hostEdit->text() );

		if ( !m_oneShare->isChecked() )
      m_loginData->currentItem()->setText( 2, "*" );
    else if (  m_oneShare->isChecked() && !m_shareEdit->text().isEmpty() )
      m_loginData->currentItem()->setText( 2, m_shareEdit->text() );

		if ( !m_loginEdit->text().isEmpty() )
      m_loginData->currentItem()->setText( 3, m_loginEdit->text() );

		if ( !m_passwdEdit->text().isEmpty() )
      m_loginData->currentItem()->setText( 4, m_passwdEdit->text() );
    else
		  // The user should be able to provide a null string password!
		  m_loginData->currentItem()->setText( 4, QString::null );

    clear();
    m_edit = false;
    m_workgroupEdit->setFocus();
	}
	else
	{
	  // First check, if workgroup and host are provided.
		if ( m_workgroupEdit->edited() && m_hostEdit->edited() )
		{
		  // Now check, if the rest has been filled out.
			// NOTE: The user is allowed to leave the password line edit
			// blank to provide a null string password.
			if ( ( m_shareEdit->edited() || !m_oneShare->isChecked() ) && m_loginEdit->edited() )
			{
        // Write all entries to the list view.
				if ( m_oneShare->isChecked() )
				{
				  if ( !m_workgroupEdit->text().stripWhiteSpace().isEmpty() && !m_hostEdit->text().stripWhiteSpace().isEmpty() &&
					     !m_shareEdit->text().stripWhiteSpace().isEmpty() && !m_loginEdit->text().stripWhiteSpace().isEmpty() )
					{
				    QListViewItem *item = new QListViewItem( m_loginData );
            item->setText( 0, m_workgroupEdit->text().upper() );
            item->setText( 1, m_hostEdit->text().upper() );
            item->setText( 2, m_shareEdit->text() );
            item->setText( 3, m_loginEdit->text() );
						if ( !m_passwdEdit->text().stripWhiteSpace().isEmpty() )
						  item->setText( 4, m_passwdEdit->text() );
					  else
              item->setText( 4, QString::null );
							
						clear();
					}
					else
            // FIXME: This can be made more specific. Tell the user which field he hasn't filled out.
		        KMessageBox::sorry( this, i18n( "You forgot to enter some data." ), QString::null, KMessageBox::Notify );
				}
				else
				{
				  if ( !m_workgroupEdit->text().stripWhiteSpace().isEmpty() && !m_hostEdit->text().stripWhiteSpace().isEmpty() &&
					     !m_loginEdit->text().stripWhiteSpace().isEmpty() )
					{
            QListViewItem *item = new QListViewItem( m_loginData );
            item->setText( 0, m_workgroupEdit->text().upper() );
            item->setText( 1, m_hostEdit->text().upper() );
            item->setText( 2, "*" );
            item->setText( 3, m_loginEdit->text() );
						if ( !m_passwdEdit->text().stripWhiteSpace().isEmpty() )
						  item->setText( 4, m_passwdEdit->text() );
					  else
              item->setText( 4, QString::null );

            clear();
				  }
					else
            // FIXME: This can be made more specific. Tell the user which field he hasn't filled out.
		        KMessageBox::sorry( this, i18n( "You forgot to enter some data." ), QString::null, KMessageBox::Notify );
				}
			}
			else
		    // FIXME: This can be made more specific. Tell the user which field he hasn't filled out.
		    KMessageBox::sorry( this, i18n( "You forgot to enter some data." ), QString::null, KMessageBox::Notify );
		}
		else
		  // FIXME: This can be made more specific. Tell the user which field he hasn't filled out. Also do a
			// check here, if other fields are also left blank!
		  KMessageBox::sorry( this, i18n( "You forgot to enter some data." ), QString::null, KMessageBox::Notify );

		m_workgroupEdit->setFocus();
	}
}


void Smb4KAuthOptions::slotClearClicked()
{
  clear();
  m_workgroupEdit->setFocus();
}


void Smb4KAuthOptions::slotDeleteClicked()
{
  if ( m_loginData->currentItem() != 0 && m_loginData->currentItem()->isSelected() )
    delete m_loginData->currentItem();
}


void Smb4KAuthOptions::slotEditClicked()
{
  m_workgroupEdit->setText( m_loginData->currentItem()->text( 0 ) );
  m_workgroupEdit->setEdited( true );
  m_hostEdit->setText( m_loginData->currentItem()->text( 1 ) );
  m_hostEdit->setEdited( true );
  if ( m_loginData->currentItem()->text( 2 ).stripWhiteSpace() == "*" )
  {
    m_oneShare->setChecked( false );
    m_shareEdit->clear();
  }
  else
  {
    m_oneShare->setChecked( true );
    m_shareEdit->setText( m_loginData->currentItem()->text( 2 ) );
    m_shareEdit->setEdited( true );
  }
  m_loginEdit->setText( m_loginData->currentItem()->text( 3 ) );
  m_loginEdit->setEdited( true );
  m_passwdEdit->setText( m_loginData->currentItem()->text( 4 ) );
  m_passwdEdit->setEdited( true );

  m_edit = true;
}


void Smb4KAuthOptions::slotContextMenu( QListViewItem *item, const QPoint &point, int col )
{
  m_col = col;
  m_menu = new KPopupMenu( this );

  QIconSet edit( SmallIcon( "edit" ), QIconSet::Automatic );
  m_menu->insertItem( edit, i18n( "Edit" ), this, SLOT( slotEditClicked() ), 0, Edit, 1 );
  
  QIconSet remove( SmallIcon( "remove" ), QIconSet::Automatic );
  m_menu->insertItem( remove, i18n( "Delete" ), this, SLOT( slotDeleteClicked() ), 0, Remove, 2 );
  
  if ( item && col < m_loginData->columns() )
  {
    m_menu->setItemEnabled( Remove, true );
    m_menu->setItemEnabled( Edit, true );
  }
  else
  {
    m_menu->setItemEnabled( Remove, false );
    m_menu->setItemEnabled( Edit, false );
  }
  m_menu->exec( point, 0 );
}


void Smb4KAuthOptions::slotShareStateChanged( int state )
{
  if ( state == QCheckBox::On )
    m_shareEdit->setEnabled( true );
  else if ( state == QCheckBox::Off )
    m_shareEdit->setEnabled( false );
}


void Smb4KAuthOptions::slotAuthDataStateChanged( int state )
{
  if ( state == QCheckBox::On )
    m_authWidget->setEnabled( true );
  else if ( state == QCheckBox::Off )
    m_authWidget->setEnabled( false );
}


#include "smb4kauthoptions.moc"
