/***************************************************************************
    smb4koptionsdlg.cpp  -  description
                             -------------------
    begin                : Sam Feb 22 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qtimer.h>

// KDE includes
#include <klocale.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kstandarddirs.h>

// application specific includes
#include "smb4koptionsdlg.h"


Smb4KOptionsDlg::Smb4KOptionsDlg( QWidget *parent, const char *name, bool modal ) 
: KDialogBase( IconList, i18n( "Configuration" ), Ok|Apply|Cancel, Ok, parent, name, modal, true )
{
  setWFlags( Qt::WDestructiveClose );
  
  m_config = kapp->config();

  setupAppearanceOptions();
  setupNetworkOptions();
  setupShareOptions();
  setupAuthOptions();
  
  readOptions();
}


Smb4KOptionsDlg::~Smb4KOptionsDlg()
{
}


void Smb4KOptionsDlg::setupNetworkOptions()
{
  QFrame *frame = addPage( i18n( "Network" ), i18n( "Network" ), DesktopIcon( "network" ) );

  QGridLayout *frameLayout = new QGridLayout( frame );
  frameLayout->setSpacing( 0 );
  frameLayout->setMargin( 0 );

  m_networkPage = new Smb4KNetworkOptions( frame );
  QSpacerItem *spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Preferred );

  frameLayout->addWidget( m_networkPage, 0, 0 );
  frameLayout->addMultiCell( spacer, 1, 10, 0, 0 );
}


void Smb4KOptionsDlg::setupShareOptions()
{
  QFrame *frame = addPage( i18n( "Shares" ), i18n( "Shares" ), DesktopIcon( "hdd_mount" ) );

  QGridLayout *frameLayout = new QGridLayout( frame );
  frameLayout->setSpacing( 0 );
  frameLayout->setMargin( 0 );

  m_sharePage = new Smb4KShareOptions( frame );
  QSpacerItem *spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Preferred );

  frameLayout->addWidget( m_sharePage, 0, 0 );
  frameLayout->addMultiCell( spacer, 1, 10, 0, 0 );
}


void Smb4KOptionsDlg::setupAuthOptions()
{
  QFrame *frame = addPage( i18n( "Authentication" ), i18n( "Authentication" ), DesktopIcon( "personal" ) );

  QGridLayout *frameLayout = new QGridLayout( frame );
  frameLayout->setSpacing( 0 );
  frameLayout->setMargin( 0 );

  m_authPage = new Smb4KAuthOptions( frame );
  QSpacerItem *spacer = new QSpacerItem( 10, 0, QSizePolicy::Preferred, QSizePolicy::Expanding );
  
  frameLayout->addWidget( m_authPage, 0, 0 );
  frameLayout->addMultiCell( spacer, 1, 10, 0, 0 );
}


void Smb4KOptionsDlg::setupAppearanceOptions()
{
  // FIXME: The icon should be a different one!
  QFrame *frame = addPage( i18n( "Appearance" ), i18n( "Appearance" ), DesktopIcon( "colorize" ) );
  
  QGridLayout *frameLayout = new QGridLayout( frame );
  frameLayout->setSpacing( 0 );
  frameLayout->setMargin( 0 );
  
  m_appearancePage = new Smb4KAppearanceOptions( frame );
  QSpacerItem *spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Preferred );
  
  frameLayout->addWidget( m_appearancePage, 0, 0 );
  frameLayout->addMultiCell( spacer, 1, 10, 0, 0 );
}


bool Smb4KOptionsDlg::saveOptions()
{
  //
  // General options
  //
  m_config->setGroup( "General Options" );
  m_config->writeEntry( "Configuration Dialog Geometry", size() );
  
  //
  // Mount options
  //
  m_config->setGroup( "Mount Options" );
  // Check, if the default path ends with a slash.
  if ( m_sharePage->m_defaultPath->lineEdit()->text().endsWith( "/" ) )
    m_config->writeEntry( "Default Path", m_sharePage->m_defaultPath->lineEdit()->text() );
  else
    m_config->writeEntry( "Default Path", m_sharePage->m_defaultPath->lineEdit()->text().append( "/" ) );
  m_config->writeEntry( "Clean Up", m_sharePage->m_cleanUp->isChecked() );
  m_config->writeEntry( "Unmount All", m_sharePage->m_unmountAll->isChecked() );
  m_config->writeEntry( "Mount Recent", m_sharePage->m_mountRecent->isChecked() );
  m_config->writeEntry( "Force Lowercase", m_sharePage->m_lowercase->isChecked() );

  //
  // Browse options
  //
  m_config->setGroup( "Browse Options" );
  if ( m_networkPage->useNMB->isChecked() )
  {
    m_config->writeEntry( "Browse List", "nmblookup" );
    if ( !m_networkPage->hostName->text().stripWhiteSpace().isEmpty() )
      m_config->writeEntry( "Query Host", m_networkPage->hostName->text().stripWhiteSpace().upper() );    
  }
  else if ( m_networkPage->useHost->isChecked() )
  {
    m_config->writeEntry( "Browse List", "host" );
    m_config->writeEntry( "Query Host", m_networkPage->hostName->text().stripWhiteSpace().upper() );
  }
  
  //
  // Appearance
  //
  m_config->setGroup( "Appearance" );
  m_config->writeEntry( "Show Hidden", m_appearancePage->m_showHidden->isChecked() );
  m_config->writeEntry( "Show IPC", m_appearancePage->m_showIPC->isChecked() );
  m_config->writeEntry( "Show ADMIN", m_appearancePage->m_showAdmin->isChecked() );
  m_config->writeEntry( "Show Printer", m_appearancePage->m_showPrinter->isChecked() );
  m_config->writeEntry( "Show Type", m_appearancePage->m_showType->isChecked() );
  m_config->writeEntry( "Show Comment", m_appearancePage->m_showComment->isChecked() );  
  if ( m_appearancePage->m_shareview->isChecked() )
    m_config->writeEntry( "Display Shares", "list" );
  else
    m_config->writeEntry( "Display Shares", "icons" );
  m_config->writeEntry( "Show Mount Point", m_appearancePage->m_description->isChecked() );
  m_config->writeEntry( "Show All Shares", m_appearancePage->m_external->isChecked() );

  //
  // Authentication
  //
  m_config->setGroup( "Authentication" );
  m_config->writeEntry( "Default Authentication", m_authPage->m_defaultAuth->isChecked() );
  if ( m_authPage->m_defaultAuth->isChecked() )
  {
    QStringList defaultAuth;
    defaultAuth.append( m_authPage->m_defaultLogin->text().stripWhiteSpace() );
    defaultAuth.append( m_authPage->m_defaultPasswd->text().stripWhiteSpace() );
    sapp->passwdReader()->writeDefaultAuthData( defaultAuth );
  }
  
  int index = 0;
  QStringList authList;
  while ( m_authPage->m_loginData->itemAtIndex( index ) != 0 )
  {
    QString wg = m_authPage->m_loginData->itemAtIndex( index )->text( 0 );          
    QString host = m_authPage->m_loginData->itemAtIndex( index )->text( 1 );
    QString share = m_authPage->m_loginData->itemAtIndex( index )->text( 2 );
    QString user = m_authPage->m_loginData->itemAtIndex( index )->text( 3 );
    QString passwd = m_authPage->m_loginData->itemAtIndex( index )->text( 4 );
    QString entry( QString( "%1:%2:%3:%4:%5" ).arg( wg ).arg( host ).arg( share ).arg( user ).arg( passwd ) );
    authList.append( entry );
    index++;
  }
  
  sapp->passwdReader()->writeAuthData( authList );
  
  m_config->sync();
  return true;
}


void Smb4KOptionsDlg::readOptions()
{
  //
  // Mount options
  //
  m_config->setGroup( "Mount Options" );
  m_sharePage->m_defaultPath->lineEdit()->setText( m_config->readEntry( "Default Path", QDir::homeDirPath().append( "/smb4k/" ) ) );
  m_sharePage->m_cleanUp->setChecked( m_config->readBoolEntry( "Clean Up", true ) );
  m_sharePage->m_unmountAll->setChecked( m_config->readBoolEntry( "Unmount All", false ) );
  m_sharePage->m_mountRecent->setChecked( m_config->readBoolEntry( "Mount Recent", false ) );
  m_sharePage->m_lowercase->setChecked( m_config->readBoolEntry( "Force Lowercase", false ) );

  //
  // Browse options
  //
  m_config->setGroup( "Browse Options" );
  // Toggle the browse list buttons.
  bool winsSupport = m_config->readBoolEntry( "WINS Support", false );
  QString browselist = m_config->readEntry( "Browse List", "nmblookup" );
  QString wins =  m_config->readEntry( "WINS Server", QString::null );
  QString host = m_config->readEntry( "Query Host", wins );
  if ( browselist.stripWhiteSpace() == "nmblookup" )
  {
    m_networkPage->useNMB->setChecked( true );
    m_networkPage->hostLabel->setEnabled( false );
    m_networkPage->hostName->setEnabled( false );
    if ( !host.isEmpty() )
      m_networkPage->hostName->setText( host );
  }
  else if ( browselist.stripWhiteSpace() == "host" )
  {
    m_networkPage->useHost->setChecked( true );
    m_networkPage->hostName->setText( host );
  }
  
  if ( winsSupport )
    m_networkPage->winsYesNo->setText( wins );

  // FIXME: This is transition stuff. Remove in version >= 0.4.0.
  if ( m_config->hasKey( "Show Hidden" ) )
  {   
    m_appearancePage->m_showHidden->setChecked( m_config->readBoolEntry( "Show Hidden", true ) );
    m_config->deleteEntry( "Show Hidden" );
  }
  if ( m_config->hasKey( "Show IPC" ) )
  {
    m_appearancePage->m_showIPC->setChecked( m_config->readBoolEntry( "Show IPC", false ) );
    m_config->deleteEntry( "Show IPC" );
  }
  if ( m_config->hasKey( "Show ADMIN" ) )
  {
    m_appearancePage->m_showAdmin->setChecked( m_config->readBoolEntry( "Show ADMIN", false ) );
    m_config->deleteEntry( "Show ADMIN" );
  }
  if ( m_config->hasKey( "Show Printer" ) )
  {
    m_appearancePage->m_showPrinter->setChecked( m_config->readBoolEntry( "Show Printer", false ) );
    m_config->deleteEntry( "Show Printer" );
  }
  if ( m_config->hasKey( "Show Type" ) )
  {
    m_appearancePage->m_showType->setChecked( m_config->readBoolEntry( "Show Type", true ) );
    m_config->deleteEntry( "Show Type" );
  }
  if ( m_config->hasKey( "Show Comment" ) )
  {
    m_appearancePage->m_showComment->setChecked( m_config->readBoolEntry( "Show Comment", true ) );
    m_config->deleteEntry( "Show Comment" );
  }

  //
  // Appearance
  //
  m_config->setGroup( "Appearance" );
  
  // FIXME: This first paragraph has to be simplified (for version >= 0.4.0). The IFs have to
  // be removed.
  if ( m_config->hasKey( "Show Hidden" ) )
    m_appearancePage->m_showHidden->setChecked( m_config->readBoolEntry( "Show Hidden", true ) );
  // The program has to be notified that the toggle state changed.
  m_appearancePage->init();
  if ( m_config->hasKey( "Show IPC" ) )
    m_appearancePage->m_showIPC->setChecked( m_config->readBoolEntry( "Show IPC", false ) );
  if ( m_config->hasKey( "Show ADMIN" ) )
    m_appearancePage->m_showAdmin->setChecked( m_config->readBoolEntry( "Show ADMIN", false ) );
  if ( m_config->hasKey( "Show Printer" ) )
    m_appearancePage->m_showPrinter->setChecked( m_config->readBoolEntry( "Show Printer", false ) );
  if ( m_config->hasKey( "Show Type" ) )
    m_appearancePage->m_showType->setChecked( m_config->readBoolEntry( "Show Type", true ) );
  if ( m_config->hasKey( "Show Comment" ) )
    m_appearancePage->m_showComment->setChecked( m_config->readBoolEntry( "Show Comment", true ) );
           
  if ( m_config->readEntry( "Display Shares", "icons" ) == "icons" )
    m_appearancePage->m_shareview->setChecked( false );
  else if ( m_config->readEntry( "Display Shares", "icons" ) == "list" )
    m_appearancePage->m_shareview->setChecked( true );
  
  if ( !m_config->readBoolEntry( "Show Mount Point", false ) )
    m_appearancePage->m_description->setChecked( false );
  else
    m_appearancePage->m_description->setChecked( true );
    
  m_appearancePage->m_external->setChecked( m_config->readBoolEntry( "Show All Shares", false ) );
  
  
  //
  // General options
  //
  m_config->setGroup( "General Options" );
  // FIXME: Transition stuff: Renamed KConfig entry. Remove this 'if' stuff in version >= 0.4.0.
  if ( m_config->hasKey( "Configuration Dialog Size" ) )
    m_config->deleteEntry( "Configuration Dialog Size" );
  else if ( m_config->hasKey( "Configuration Dialog Geometry" ) )
    setInitialSize( m_config->readSizeEntry( "Configuration Dialog Geometry", 0L ), false );

  
  //
  // Authentication
  //
  m_config->setGroup( "Authentication" );
  if ( m_config->readBoolEntry( "Default Authentication", false ) )
  {
    m_authPage->m_defaultAuth->setChecked( true );
    m_authPage->m_authWidget->setEnabled( true );
  }
  else
  {
    m_authPage->m_defaultAuth->setChecked( false );
    m_authPage->m_authWidget->setEnabled( false );
  }
  
  // Now read the passwords:
  QStringList authList = sapp->passwdReader()->read();
  uint index( 0 );
  while ( index < authList.count() )
  {
    QListViewItem *item = new QListViewItem( m_authPage->m_loginData );
    item->setText( 0, authList[index].section( ":", 0, 0 ) );
    item->setText( 1, authList[index].section( ":", 1, 1 ) );
    item->setText( 2, authList[index].section( ":", 2, 2 ) );
    item->setText( 3, authList[index].section( ":", 3, 3 ) );
    item->setText( 4, authList[index].section( ":", 4, 4 ) );
    index++;
  }
  
  // And the default authentication:
  QStringList defaultAuthList = sapp->passwdReader()->readDefaultAuthData();
  m_authPage->m_defaultLogin->setText( defaultAuthList[0] );
  m_authPage->m_defaultPasswd->setText( defaultAuthList[1] );  
}


/****************************************************************************
   Checks, whether everything has been done
****************************************************************************/


bool Smb4KOptionsDlg::check()
{
  bool login, browse;
  
  //
  // Authentication
  //
  if ( m_authPage->m_hostEdit->text().length() != 0 || m_authPage->m_workgroupEdit->text().length() != 0 ||
    m_authPage->m_loginEdit->text().length() != 0 || m_authPage->m_passwdEdit->text().length() != 0 )
  {
    if ( KMessageBox::warningYesNo( this, i18n( "You have not committed some authentication data.\nDo you want to do it now?" ), QString::null, KStdGuiItem::yes(), KStdGuiItem::no(), QString::null, KMessageBox::Notify ) == KMessageBox::Yes )
      m_authPage->slotAddClicked();  
    login = true;
  }
  else
    login = true;
    
  if ( m_authPage->m_defaultAuth->isChecked() &&  m_authPage->m_defaultLogin->text().stripWhiteSpace().isEmpty() ) 
  {
	  // FIXME: The string should be: "You have not entered at least the default user name.\nDo you want to do it now?"
    if ( KMessageBox::warningYesNo( this, i18n( "You have not entered the default authentication data completely.\nDo you want to do it now?" ),
    QString::null, KStdGuiItem::yes(), KStdGuiItem::no(), QString::null, KMessageBox::Notify ) == KMessageBox::Yes )
    {
      showPage( pageIndex( m_authPage ) );
      login = false;
    }
    else
    {
      m_authPage->m_defaultAuth->setChecked( false );
      m_authPage->m_defaultLogin->clear();
      m_authPage->m_defaultPasswd->clear();
      login = true;
    }
  }
    
  //
  // Network
  //
  if ( m_networkPage->hostName->text().stripWhiteSpace().isEmpty() && m_networkPage->hostName->isEnabled() )
  {
    if ( KMessageBox::warningYesNo( this, i18n( "Please enter a host, that you want to query." ), QString::null, KStdGuiItem::ok(), KStdGuiItem::cancel(), QString::null, KMessageBox::Notify ) == KMessageBox::Yes )
    {
      showPage( pageIndex( m_networkPage ) );
      browse = false;
    }
    else
      browse = false;
  }
  else
    browse = true;

  if ( login && browse )
    return true;
  else
    return false;
}


/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////

void Smb4KOptionsDlg::slotOk()
{
  if ( check() )
  {
    saveOptions();
    accept();
    sapp->readOptions();
  }
}


void Smb4KOptionsDlg::slotApply()
{
  if ( check() )
  {
    saveOptions();
    sapp->readOptions();
  }
}


#include "smb4koptionsdlg.moc"

