/***************************************************************************
    smb4kpreviewdialog.cpp  -  description
                             -------------------
    begin                : Fre Jul 4 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qtooltip.h>
#include <qiconset.h>
#include <qregexp.h>
 
// KDE includes
#include <klocale.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kmessagebox.h>

// application specific includes
#include "smb4kpreviewdialog.h"
#include "../../smb4k.h"


Smb4KPreviewDialog::Smb4KPreviewDialog( const QString &workgroup, const QString &host,
const QString &share, QWidget *parent, const char *name, bool modal )
: KDialogBase( Plain, i18n( "Preview" ), Close, Close, parent, name, modal, true )
{
  m_config = kapp->config();

  m_workgroup = workgroup;
  m_host = host;
  m_share = share;

  m_address = QString( "//%1/%2/" ).arg( m_host ).arg( m_share );

  setupView();

  readOptions();

  setMinimumSize( sizeHint() );
  setWFlags( Qt::WDestructiveClose );

  connect( this, SIGNAL( finished() ), SLOT( slotFinished() ) );
  connect( m_view, SIGNAL( executed( QIconViewItem * ) ), SLOT( slotItemExecuted( QIconViewItem * ) ) );
  connect( m_toolbar, SIGNAL( clicked( int ) ), SLOT( slotButtonClicked( int ) ) );
  connect( m_combo, SIGNAL( activated( const QString & ) ), SLOT( slotItemActivated( const QString & ) ) );
  
  connect( sapp->scanner(), SIGNAL( previewResult( const QStringList & ) ), this, SLOT( slotReceivedData( const QStringList & ) ) );
  connect( sapp->scanner(), SIGNAL( prev_error( const QString & ) ), this, SLOT( slotError( const QString & ) ) );
  
  sapp->scanner()->getPreview( workgroup, host, share, QString::null );
}


Smb4KPreviewDialog::~Smb4KPreviewDialog()
{
}


void Smb4KPreviewDialog::setupView()
{
  QFrame *frame = plainPage();
  QGridLayout *layout = new QGridLayout( frame );

  m_view = new KIconView( frame, 0, 0 );
  m_view->setItemTextPos( KIconView::Right );
  m_view->setResizeMode( KIconView::Adjust );
  m_view->setArrangement( KIconView::TopToBottom );
  m_view->setSpacing( 1 );
  m_view->setGridX( 200 );
  m_view->setWordWrapIconText( false );
  m_view->setShowToolTips( true );
  m_view->setAutoArrange( true );
  m_view->setSorting( true, true );

  m_toolbar = new KToolBar( frame, 0, true, false );
  m_toolbar->insertButton( BarIcon( "up" ), Up, true, i18n( "Up" ), 0 );
  m_combo = new KComboBox( false, m_toolbar, 0 );
  m_combo->setDuplicatesEnabled( false );
  m_toolbar->insertWidget( Combo, 10, m_combo, 2 );
  m_toolbar->setItemAutoSized( Combo, true );

  layout->addWidget( m_view, 0, 0, 0 );
  layout->addWidget( m_toolbar, 1, 0, 0 );
  setTooltips();
}


void Smb4KPreviewDialog::saveOptions()
{
  m_config->setGroup( "General Options" );
  m_config->writeEntry( "Preview Dialog Geometry", size() );
}


void Smb4KPreviewDialog::readOptions()
{
  m_config->setGroup( "General Options" );
  
  // FIXME: Transition stuff: Renamed KConfig entry. Remove this 'if' stuff in version 0.4.0.
  if ( m_config->hasKey( "Preview Dialog Size" ) )
  {
    setInitialSize( m_config->readSizeEntry( "Preview Dialog Size", 0L ) );
    m_config->deleteEntry( "Preview Dialog Size" );
  }
  else
    setInitialSize( m_config->readSizeEntry( "Preview Dialog Geometry", 0L ) );
}


void Smb4KPreviewDialog::setTooltips()
{
  QToolTip::add( m_view, i18n( "Contents" ) );
}


/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////


void Smb4KPreviewDialog::slotFinished()
{
  saveOptions();
}


void Smb4KPreviewDialog::slotReceivedData( const QStringList &data )
{
  m_dirNum = 0;
  m_fileNum = 0;
  
  m_view->clear();
  
  QString item = QString( "%1%2" ).arg( m_address ).arg( m_path );

  if ( m_combo->count() == 0 )
  {
    m_combo->insertItem( item, -1 );
    m_combo->setCurrentItem( 0 );
  }
  else if ( !m_combo->contains( item ) )
  {
    m_combo->insertItem( item, 0 );
    m_combo->setCurrentItem( 0 );
  }
  else
  {
    int index = 0;
    int pos = 0;
    bool exists = false;
    while ( index < m_combo->count() )
    {
      if ( m_combo->text( index ).stripWhiteSpace() == item.stripWhiteSpace() )
      {
        pos = index;
        exists = true;
        break;
      }
      else if ( m_combo->text( index ).stripWhiteSpace() == m_address )
      {
        pos = index;
        exists = true;
        break;
      }
      index++;
    }
    if ( !exists )
    {
      m_combo->insertItem( item, 0 );
      m_combo->setCurrentItem( 0 );
    }
    else
      m_combo->setCurrentItem( pos );
  }
  
  // OK, process the whole stuff.
  uint index( 0 );
  while ( index < data.count() )
  {
    if ( data[index].stripWhiteSpace().startsWith( "Domain" ) || data[index].stripWhiteSpace().startsWith( "OS" )
      || data[index].stripWhiteSpace().startsWith( "Anonymous" ) )
      index++;
    else if ( data[index].contains( "blocks of size" ) )
      break;
    else
    {
      QString line = data[index].stripWhiteSpace();
      if ( !line.stripWhiteSpace().startsWith( ". " )
            && !line.stripWhiteSpace().startsWith( ".. " )
            && !line.stripWhiteSpace().startsWith( "[" ) )
      {
        QString dir, file;
        QRegExp exp1( "  D..  " );
        if ( line.find( exp1, 0 ) != -1 )
        {
          dir = line.section( exp1, 0, 0 ).stripWhiteSpace();
          QIconViewItem *item = new QIconViewItem( m_view );
          item->setKey( QString( "0%1" ).arg( dir ) );
          
          // If the folder_green icon is not available, use the default KDE folder
	  // icon instead:
          KIconLoader *loader = kapp->iconLoader();
          if ( !loader->iconPath( "folder_green", KIcon::Small, true ).isEmpty() )
            item->setPixmap( SmallIcon( "folder_green" ) );
          else
            item->setPixmap( SmallIcon( "folder" ) );
          item->setText( dir );
          m_dirNum++;
        }
        else
        {
          QRegExp exp2( "  ...  " );
          file = line.section( exp2, 0, 0 ).stripWhiteSpace();
          QIconViewItem *item = new QIconViewItem( m_view );
          item->setKey( QString( "1%1" ).arg( file ) );
          item->setPixmap( SmallIcon( "unknown" ) );
          item->setText( file );
          m_fileNum++;
        }
      }       
    }
    index++;
  }
}


void Smb4KPreviewDialog::slotItemExecuted( QIconViewItem *item )
{
  if ( !item->key().startsWith( "1" ) )
  {
    QString tmp = item->text();
    m_path.append( tmp ).append( "/" );
    
    sapp->scanner()->getPreview( m_workgroup, m_host, m_share, m_path );
  }
}


void Smb4KPreviewDialog::slotButtonClicked( int id )
{
  if ( id == Up && !m_path.isEmpty() )
  {
    if ( m_path.contains( "/" ) > 1 )
      m_path = m_path.section( "/", 0, -3 ).append( "/" );
    else if ( m_path.contains( "/", true ) == 1 )
      m_path = QString::null;

    sapp->scanner()->getPreview( m_workgroup, m_host, m_share, m_path ); 
  }
}


void Smb4KPreviewDialog::slotItemActivated( const QString &item )
{
  // First we have to strip the address:
  QString path = item.section( m_address, 1, 1 ).stripWhiteSpace(); 
  
  sapp->scanner()->getPreview( m_workgroup, m_host, m_share, path );
  
  m_path = path;
}


void Smb4KPreviewDialog::slotError( const QString &message )
{
  if ( message.contains( "NT_STATUS_NO_SUCH_FILE" ) == 0 )
    KMessageBox::detailedError( this, i18n( "An error occurred." ), message, QString::null, KMessageBox::Notify );
  else
  {
    KMessageBox::error( this, QString( i18n( "Could not open directory." ) ), QString::null, KMessageBox::Notify );
    if ( m_path.contains( "/" ) == 1 )
      m_path = QString::null;
    else
      m_path = m_path.section( "/", 0, -2 ).stripWhiteSpace();
  }
}


#include "smb4kpreviewdialog.moc"

