/***************************************************************************
    smb4kbrowserwidget.cpp  -  Network browser widget class of Smb4K.
                             -------------------
    begin                : Sam Feb 22 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kapplication.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>

// application specific includes
#include "smb4kbrowserwidget.h"
#include "../core/smb4kscanner.h"
#include "../core/smb4kmounter.h"
#include "../smb4k.h"
#include "preview/smb4kpreviewdialog.h"


Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KListView( parent, name )
{
  m_config = kapp->config();

  setFrameShape( KListView::PopupPanel );
  setFrameShadow( KListView::Sunken );
  setRootIsDecorated( true );

  addColumn( i18n( "Network" ), -1 );
  addColumn( i18n( "Type" ), -1 );
  addColumn( i18n( "Comment" ), -1 );

  m_menu = new KPopupMenu( this );
  m_scanner = false;
  m_menu = new KPopupMenu( this );

  // Widget specific connections.
  connect( this, SIGNAL( executed( QListViewItem * ) ), SLOT( slotItemExecuted( QListViewItem * ) ) );
  connect( this, SIGNAL( expanded( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
  connect( this, SIGNAL( collapsed( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
  connect( this, SIGNAL( selectionChanged( QListViewItem * ) ), SLOT( slotSelectionChanged( QListViewItem * ) ) );
  connect( this, SIGNAL( clicked( QListViewItem * ) ), SLOT( slotItemClicked( QListViewItem * ) ) );
  connect( this, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint &, int ) ), SLOT( slotRightButtonPressed( QListViewItem *, const QPoint &, int ) ) );
  
  // External connections.
  connect( sapp->scanner(), SIGNAL( workgroups( const QStringList & ) ), this, SLOT( slotWorkgroups( const QStringList & ) ) );
  connect( sapp->scanner(), SIGNAL( members( const QString &, const QStringList & ) ), this, SLOT( slotMembers( const QString &, const QStringList & ) ) );
  connect( sapp->scanner(), SIGNAL( shares( const QString &, const QString &, const QStringList & ) ), this, SLOT( slotShares( const QString &, const QString &, const QStringList & ) ) );
  connect( sapp->scanner(), SIGNAL( running( bool ) ), this, SLOT( slotRunning( bool ) ) );
  connect( sapp->scanner(), SIGNAL( scan_error( const QString & ) ), this, SLOT( slotError( const QString & ) ) );
}


Smb4KBrowserWidget::~Smb4KBrowserWidget()
{
}


void Smb4KBrowserWidget::setOpen( QListViewItem *item, bool open )
{
  if ( open && item->depth() == 0 )
  {
    // Avoid problems with added hosts, that introduce a new workgroup:
    if ( m_groupData.grep( item->text( 0 ) ).count() != 0 )
    {
      // Find the current workgroup in the list and determine the master afterwards.
      QStringList::ConstIterator it = m_groupData.find( item->text( 0 ) );
      // The master:
      it--;
      sapp->scanner()->getGroupMembers( item->text( 0 ), *it );
    }
    else
    {
      // Append the workgroup and the pseudo-master to the list.
      // First the master:
      m_groupData.append( item->firstChild()->text( 0 ) );
      // Second the workgroup:
      m_groupData.append( item->text( 0 ) );
      sapp->scanner()->getGroupMembers( item->text( 0 ), item->firstChild()->text( 0 ) );
    }    
  }
  else if ( open && item->depth() == 1 && item->childCount() == 0 )
  {
    sapp->scanner()->getShares( item->parent()->text( 0 ), item->text( 0 ) );
  }
  KListView::setOpen( item, open );
}


void Smb4KBrowserWidget::insertItem( const QString &workgroup, const QString &host, const QString &comment )
{
  QListViewItem *workgroupItem = findItem( workgroup, 0, ExactMatch|CaseSensitive );
  if ( workgroupItem != 0 )
  {
    QListViewItem *hostItem = findItem( host, 0, ExactMatch|CaseSensitive );
    if ( hostItem == 0 )
    {
      QListViewItem *newItem = new QListViewItem( workgroupItem, host, QString::null, comment );
      newItem->setExpandable( true );
      newItem->setPixmap( 0, SmallIcon( "network_local" ) );
    }
    else if ( hostItem != 0 && hostItem->parent()->text( 0 ) != workgroup )
    {
      QListViewItem *newItem = new QListViewItem( workgroupItem, host, QString::null, comment );
      newItem->setExpandable( true );
      newItem->setPixmap( 0, SmallIcon( "network_local" ) );
    }
    else
      KMessageBox::sorry( this, i18n( "This computer is already in the list!" ), QString::null, KMessageBox::Notify );
  }
  else
  {
    QListViewItem *item = new QListViewItem( this, workgroup );
    item->setExpandable( true );
    item->setPixmap( 0, SmallIcon( "network" ) );
    QListViewItem *hostItem = new QListViewItem( item, host, QString::null, comment );
    hostItem->setExpandable( true );
    hostItem->setPixmap( 0, SmallIcon( "network_local" ) );
  }
}


void Smb4KBrowserWidget::readOptions()
{
  // Read the configuration.
  m_config->setGroup( "Appearance" );
  m_showType = m_config->readBoolEntry( "Show Type", true );
  m_showComment = m_config->readBoolEntry( "Show Comment", true );

#if QT_VERSION >= 0x030200
  // Now put everything back in according to the wishes of the user.
  if ( m_showType && m_showComment )
  {
    setColumnWidth( 1, 10 );
    setColumnWidth( 2, 10 );
    setColumnWidthMode( 1, QListView::Maximum );
    setColumnWidthMode( 2, QListView::Maximum );
  }
  if ( m_showType && !m_showComment )
  {
    setColumnWidth( 1, 10 );
    setColumnWidth( 2, 0 );
    setColumnWidthMode( 1, QListView::Maximum );
    setColumnWidthMode( 2, QListView::Manual );
  }
  else if ( !m_showType && m_showComment )
  {
    setColumnWidth( 1, 0 );
    setColumnWidth( 2, 10 );
    setColumnWidthMode( 1, QListView::Manual );
    setColumnWidthMode( 2, QListView::Maximum );
  }
  else if ( !m_showType && !m_showComment )
  {
    setColumnWidth( 1, 0 );
    setColumnWidth( 2, 0 );
    setColumnWidthMode( 1, QListView::Manual );
    setColumnWidthMode( 2, QListView::Manual );
  }

  // Adjust the columns. Avoid those that have a width of 0.
  int index = 0;
  while ( index < this->columns() )
  {
    if ( this->columnWidth( index ) != 0 )
      adjustColumn( index );

    index++;
  }
#else
   // Now put everything back in according to the wishes of the user.
  if ( m_showType && m_showComment )
  {
    QFontMetrics *fm = new QFontMetrics( QFont() );
    setColumnWidth( 1, fm->width( columnText( 1 ), -1 ) + 2*itemMargin() + fm->width( "w" ) );
    setColumnWidth( 2, fm->width( columnText( 2 ), -1 ) + 2*itemMargin() + fm->width( "w" ) );
    setColumnWidthMode( 1, QListView::Maximum );
    setColumnWidthMode( 2, QListView::Maximum );
  }
  if ( m_showType && !m_showComment )
  {
    QFontMetrics *fm = new QFontMetrics( QFont() );
    setColumnWidth( 1, fm->width( columnText( 1 ), -1 ) + 2*itemMargin() + fm->width( "w" ) );
    setColumnWidth( 2, 0 );
    setColumnWidthMode( 1, QListView::Maximum );
    setColumnWidthMode( 2, QListView::Manual );
  }
  else if ( !m_showType && m_showComment )
  {
    QFontMetrics *fm = new QFontMetrics( QFont() );
    setColumnWidth( 1, 0 );
    setColumnWidth( 2, fm->width( columnText( 2 ), -1 ) + 2*itemMargin() + fm->width( "w" ) );
    setColumnWidthMode( 1, QListView::Manual );
    setColumnWidthMode( 2, QListView::Maximum );
  }
  else if ( !m_showType && !m_showComment )
  {
    setColumnWidth( 1, 0 );
    setColumnWidth( 2, 0 );
    setColumnWidthMode( 1, QListView::Manual );
    setColumnWidthMode( 2, QListView::Manual );
  }

  // Adjust the widths of the columns.
  for ( int index = 0; index < childCount(); index++ )
  {
    itemAtIndex( index )->widthChanged( -1 );
  }
#endif

  // Initialize the rest.
  m_hidden = m_config->readBoolEntry( "Show Hidden", true );
  m_ipc = m_config->readBoolEntry( "Show IPC", false );
  m_admin = m_config->readBoolEntry( "Show ADMIN", false );
  m_printer = m_config->readBoolEntry( "Show Printer", false );
}


/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////


void Smb4KBrowserWidget::slotItemExecuted( QListViewItem *item )
{
  if ( item && item->depth() == 2 )
  {
    if ( item->text( 0 ).stripWhiteSpace() == "homes" )
    {
      qWarning( "INFO [Smb4KBrowserWidget::slotItemExecuted()]: Special handling of\nhomes shares has not been implemented yet. For now, Smb4K will\nhandle it like a regular one." );
      if ( item->text( 1 ).contains( "IPC" ) == 0 &&  item->text( 1 ).contains( "Printer" ) == 0 )
        sapp->mounter()->mountShare( item->parent()->parent()->text( 0 ), item->parent()->text( 0 ), item->text( 0 ) );
    }
    // Since Smb4K 0.3.1rc, the "Locked" string was removed from this query.
    else if ( item->text( 1 ).contains( "IPC" ) == 0 &&  item->text( 1 ).contains( "Printer" ) == 0 )
      sapp->mounter()->mountShare( item->parent()->parent()->text( 0 ), item->parent()->text( 0 ), item->text( 0 ) );
    else if ( item->text( 1 ).contains( "Printer" ) != 0 )
       KMessageBox::sorry( this, i18n( "Smb4K can't handle printer shares yet." ), QString::null, KMessageBox::Notify );
  }
}


void Smb4KBrowserWidget::slotItemExpandedCollapsed( QListViewItem *item )
{
  setSelected( item, true );
}


void Smb4KBrowserWidget::slotSelectionChanged( QListViewItem *item )
{
  if ( item && item->depth() == 2 )
  {
    sapp->networkPreview->setEnabled( true );
    sapp->sharesMount->setEnabled( true );
    sapp->networkAskPass->setEnabled( true );
  }
  else if ( item && item->depth() == 1 )
  {
    sapp->networkAskPass->setEnabled( true );
    sapp->networkPreview->setEnabled( false);
    sapp->sharesMount->setEnabled( false );
  }
  else
  {
    sapp->networkPreview->setEnabled( false );
    sapp->sharesMount->setEnabled( false );
    sapp->networkAskPass->setEnabled( false );
  }
  
  if ( item )
    emit highlightedItem( item );
}


void Smb4KBrowserWidget::slotRightButtonPressed( QListViewItem *item, const QPoint &point, int col )
{
  m_menu->clear();

  m_menu->insertTitle( SmallIcon( "network" ), i18n( "Network" ), Title, 0 );
  
  QIconSet reloadSet( SmallIcon( "reload" ), QIconSet::Automatic );
  m_menu->insertItem( reloadSet, i18n( "Rescan" ), sapp->scanner(), SLOT( slotInit() ), i18n( "CTRL+R" ), Rescan, 1 );

  QIconSet abortSet( SmallIcon( "stop" ), QIconSet::Automatic );
  m_menu->insertItem( abortSet, i18n( "Abort" ), sapp, SLOT( slotAbort() ), i18n( "CTRL+A" ), Abort, 2 );

  m_menu->insertSeparator( 3 );

  QIconSet searchSet( SmallIcon( "find" ), QIconSet::Automatic );
  m_menu->insertItem( searchSet, i18n( "Search" ), sapp->view(), SLOT( slotShowSearchTab() ), i18n( "CTRL+S" ), Search, 4 );

  m_menu->insertSeparator( 5 );

  QIconSet authSet( SmallIcon( "personal" ), QIconSet::Automatic );
  m_menu->insertItem( authSet, i18n( "Authentication" ), this, SLOT( slotAskPass() ), i18n( "CTRL+T" ), Password, 6 );

  QIconSet previewSet( SmallIcon( "view_icon" ), QIconSet::Automatic );
  m_menu->insertItem( previewSet, i18n( "Preview" ), this, SLOT( slotPreview() ), i18n( "CTRL+V" ), Preview, 7 );

  QIconSet mountSet( SmallIcon( "hdd_mount" ), QIconSet::Automatic );
  m_menu->insertItem( mountSet, i18n( "Mount" ), this, SLOT( slotMountShare() ), i18n( "CTRL+M" ), Mount, 8 );
  
  // Enable and disable menu items.
  if ( m_scanner )
  {
    m_menu->setItemEnabled( Rescan, false );
    m_menu->setItemEnabled( Abort, true );
  }
  else
  {
    m_menu->setItemEnabled( Rescan, true );
    m_menu->setItemEnabled( Abort, false );
  }
  if ( item && item->depth() == 2 && col < columns() )
  {
    m_menu->setItemEnabled( Preview, true );
    m_menu->setItemEnabled( Mount, true );
    m_menu->setItemEnabled( Password, true );
  }
  else if ( item && item->depth() == 1 && col < columns() )
  {
    m_menu->setItemEnabled( Password, true );
    m_menu->setItemEnabled( Preview, false );
    m_menu->setItemEnabled( Mount, false );
  }
  else
  {
    m_menu->setItemEnabled( Preview, false );
    m_menu->setItemEnabled( Mount, false );
    m_menu->setItemEnabled( Password, false );
  }
  
  m_menu->exec( point, 0 );
}


void Smb4KBrowserWidget::slotItemClicked( QListViewItem *item )
{
  // We only need this for the case, that the user clicks somewhere, where
  // NO item is. Smb4KNetworkTab needs it to clear the tab.
  if ( !item && !selectedItem() )
    emit highlightedItem( item );
}


void Smb4KBrowserWidget::slotWorkgroups( const QStringList &list )
{
  // Tell the rest of the program (i.e. the network tab), that new data arrived.
  emit rescanned();
  
  m_groupData.clear();
  m_groupData = list;
  clear();
  uint index( 0 );
  // Put the workgroups to the list view.
  if ( !m_groupData[0].stripWhiteSpace().isEmpty() && !m_groupData[1].stripWhiteSpace().isEmpty() )
  {
    while ( index < m_groupData.count() )
    {
      index++;
      if ( findItem( m_groupData[index].stripWhiteSpace(), 0, CaseSensitive|ExactMatch ) != 0 )
      {
        index++;
        continue;
      }
      else
      {
        QListViewItem *item = new QListViewItem( this, m_groupData[index++].stripWhiteSpace() );
        item->setExpandable( true );
        item->setPixmap( 0, SmallIcon( "network" ) );
      }
    }
  }
}


void Smb4KBrowserWidget::slotMembers( const QString &workgroup, const QStringList &list )
{
  // If the list is not empty, put everything in the list view:
  if ( list.count() != 0 )
  {
    uint index( 0 );
    while ( index < list.count() )
    {
      QString line = list[index++].stripWhiteSpace();
      // Split the string in half.
      QString host = line.left( line.find( "  ", 0 ) ).stripWhiteSpace();
      QString comment = line.right( line.length() - line.find( "  ", 0 ) ).stripWhiteSpace();
      // Give the master its comment.
      if ( findItem( host, 0, CaseSensitive|ExactMatch ) != 0 )
      {
        QListViewItem *masterItem = findItem( host, 0, CaseSensitive|ExactMatch );
        masterItem->setText( 2, comment );
      }
      // Put the other hosts with their comments in the list.
      else
      {
        if ( findItem( host, 0, CaseSensitive|ExactMatch ) != 0 )
          continue;
        else
        {
          QListViewItem *hostItem = new QListViewItem( findItem( workgroup, 0, ExactMatch|CaseSensitive ), host, QString::null, comment );
          hostItem->setExpandable( true );
          hostItem->setPixmap( 0, SmallIcon( "network_local" ) );
        }
      }
    }
    // We want to make sure, that the master is also in the list.
    QStringList::ConstIterator it = m_groupData.find( workgroup );
    it--;
    if ( findItem( (*it).stripWhiteSpace(), 0, ExactMatch|CaseSensitive ) == 0 )
    {
      QListViewItem *masterItem = new QListViewItem( findItem( workgroup, 0, ExactMatch|CaseSensitive ), (*it).stripWhiteSpace() );
      masterItem->setExpandable( true );
      masterItem->setPixmap( 0, SmallIcon( "network_local" ) );
    }     
  }
  else
  {
    QStringList::ConstIterator it = m_groupData.find( workgroup );
    it--;
    QListViewItem *item = findItem( (*it).stripWhiteSpace(), 0, ExactMatch|CaseSensitive );
    if ( item == 0 )
    {
      QListViewItem *masterItem = new QListViewItem( findItem( workgroup, 0, ExactMatch|CaseSensitive ), (*it).stripWhiteSpace() );
      masterItem->setExpandable( true );
      masterItem->setPixmap( 0, SmallIcon( "network_local" ) );
    }
    else if ( item != 0 && item->parent()->text( 0 ) != workgroup )
    {
      QListViewItem *masterItem = new QListViewItem( findItem( workgroup, 0, ExactMatch|CaseSensitive ), (*it).stripWhiteSpace() );
      masterItem->setExpandable( true );
      masterItem->setPixmap( 0, SmallIcon( "network_local" ) );
    }
  }
}


void Smb4KBrowserWidget::slotShares( const QString &workgroup, const QString &host, const QStringList &list )
{
  // The program has to do nothing, if the list is empty. Test:
  if ( !list.isEmpty() )
  {
    // Now find the host. 
    QListViewItem *hostItem = findItem( workgroup, 0, ExactMatch|CaseSensitive )->firstChild();
    
    if ( hostItem )
    {
      while ( hostItem )
      {
        if ( hostItem->text( 0 ) == host )
          break;
        else
          hostItem = hostItem->nextSibling();      
      }
      
      // Now that we have found the host, we can process the list.
      for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it )
      {
        QString share, type, comment;
        
        // We have to split the lines:
        if ( (*it).contains( "Disk" ) )
        {
          share = (*it).section( "Disk", 0, 0 ).stripWhiteSpace();
          type = "Disk";
          comment = (*it).section( "Disk", 1, 1 ).stripWhiteSpace();
        }
        else if ( (*it).contains( "Printer" ) )
        {
          share = (*it).section( "Printer", 0, 0 ).stripWhiteSpace();
          type = "Printer";
          comment = (*it).section( "Printer", 1, 1 ).stripWhiteSpace();
        }
        else
        {
          share = (*it).section( ' ', 0, 0 ).stripWhiteSpace();
          type = (*it).section( ' ', 1, 1 ).stripWhiteSpace();
          comment = (*it).section( ' ', 2, -1 ).stripWhiteSpace();
        }
          
        // Before displaying the share, it has to be checked whether the user
        // wants this share to be displayed at all:

        // Take care of the hidden shares.
        if ( !m_hidden && share.endsWith( "$" ) )
          continue;
        else
        {
          // Show hidden shares but ignore IPC$ shares.
          if ( !m_ipc && share.contains( "IPC" ) != 0 )
            continue;
          // Show hidden shares but ignore ADMIN$ shares
          if ( !m_admin && share.contains( "ADMIN" ) != 0 )
            continue;
        }
        
        // If the user does not want the printer shares to be displayed,
        // they will be ignored here.
        if ( !m_printer && type.contains( "Printer" ) != 0 )
          continue;  
          
        // We are prepared to display the share, but one last check has to be done:
        QListViewItem *test = findItem( share, 0, ExactMatch|CaseSensitive );
        if ( test == 0 || test->parent() != hostItem )
        {
          QListViewItem *item = new QListViewItem( hostItem, share );

          // Now process the entries in the list.
          if ( type.contains( "Printer" ) != 0 )
          {
            item->setPixmap( 0, SmallIcon( "printer2" ) );
            item->setText( 1, type );
            item->setText( 2, comment );
          }
          else
          {
      	    // If the folder_green icon is not available, use the default KDE folder
	    // icon instead:
            KIconLoader *loader = kapp->iconLoader();
            if ( !loader->iconPath( "folder_green", KIcon::Small, true ).isEmpty() )
              item->setPixmap( 0, SmallIcon( "folder_green" ) );
            else
              item->setPixmap( 0, SmallIcon( "folder" ) );
            item->setText( 1, type );
            item->setText( 2, comment );          
          }
        }    
      }
    }
  }
}


void Smb4KBrowserWidget::slotRunning( bool on )
{
  m_scanner = on;

  // If the menu is shown, enable or disable the
  // items on the fly.
  if ( m_menu->isShown() )
  {
    if ( m_scanner )
    {
      m_menu->setItemEnabled( Rescan, false );
      m_menu->setItemEnabled( Abort, true );
    }
    else
    {
      m_menu->setItemEnabled( Rescan, true );
      m_menu->setItemEnabled( Abort, false );
    }
  }  
}


void Smb4KBrowserWidget::slotError( const QString &message )
{
  KMessageBox::detailedError( this, i18n( "An error occurred." ), message, QString::null, KMessageBox::Notify );
}


void Smb4KBrowserWidget::slotMountShare()
{
  if ( currentItem() && currentItem()->depth() == 2 )
  {
    QString workgroup = currentItem()->parent()->parent()->text( 0 );
    QString host =  currentItem()->parent()->text( 0 );
    QString share = currentItem()->text( 0 );
    
    sapp->mounter()->mountShare( workgroup, host, share );
  }
}


void Smb4KBrowserWidget::slotAskPass()
{
  if ( currentItem() && currentItem()->depth() == 1 )
  {
    QString workgroup = currentItem()->parent()->text( 0 );
    QString host = currentItem()->text( 0 );
    QString share = QString::null;
    
    sapp->passwdReader()->askpass( workgroup, host, share );
  }
  else if ( currentItem() && currentItem()->depth() == 2 )
  {
    QString workgroup = currentItem()->parent()->parent()->text( 0 );
    QString host = currentItem()->parent()->text( 0 );
    QString share = currentItem()->text( 0 );

    sapp->passwdReader()->askpass( workgroup, host, share );
  }
}


void Smb4KBrowserWidget::slotPreview()
{
  QString workgroup = currentItem()->parent()->parent()->text( 0 );
  QString host = currentItem()->parent()->text( 0 );
  QString share = currentItem()->text( 0 );
  
  Smb4KPreviewDialog *preview = new Smb4KPreviewDialog( workgroup, host, share, this, 0, false );
  preview->show();
}


#include "smb4kbrowserwidget.moc"
