/***************************************************************************
    smb4kbrowserwidget.h  -  Network browse widget class for Smb4K.
                             -------------------
    begin                : Sam Feb 22 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4KBROWSERWIDGET_H
#define SMB4KBROWSERWIDGET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qstringlist.h>

// KDE includes
#include <klistview.h>
#include <kconfig.h>
#include <kpopupmenu.h>

// forward declarations
class Smb4KScanner;

/** This is the browser widget class of Smb4K. */

class Smb4KBrowserWidget : public KListView
{
  Q_OBJECT

  friend class Smb4KView;

  public:
    /** The constructor. */
    Smb4KBrowserWidget( QWidget *parent = 0, const char *name = 0 );
    /** The destructor. */
    ~Smb4KBrowserWidget();

    /** This function is called, when the user tries to open an item in the list view.
        Takes the selected item and a bool as arguments. For more information
        look at the class documentation of KListView. */
    void setOpen( QListViewItem *item, bool on );
    /** This function inserts a listview item into the browser window. */
    void insertItem( const QString &workgroup, const QString &host, const QString &comment );
    /** This function reads the options necessary for building up the browser widget. */
    void readOptions();

  signals:
    /** This signal emits the selected list view item. */
    void highlightedItem( QListViewItem *item );
    /** Is emitted, when the network was rescanned and new data is coming in. */
    void rescanned();

  protected:
    /** This function processes the data obtained by scanning a specific host and displays the
        available shares. */
    void processShares();

  protected slots:
    /** Is called, when a list view item is to be mounted. */
    void slotItemExecuted( QListViewItem *item );
    /** Is called whenever a QListViewItem is expanded or collapsed. */
    void slotItemExpandedCollapsed( QListViewItem *item );
    /** Is called, when the selection in the list view is changed. (Is
        connected to the selectionChanged() signal. */
    void slotSelectionChanged( QListViewItem *item );
    /** This slot is invoked by the rightButtonPressed() signal. */
    void slotRightButtonPressed( QListViewItem *item, const QPoint &point, int col );
    /** Is called, if the user clicked somewhere in the list view. */
    void slotItemClicked( QListViewItem *item );
    /** Takes the workgroups/masters list. */
    void slotWorkgroups( const QStringList &list );
    /** Takes the member list of a workgroup. */
    void slotMembers( const QString &workgroup, const QStringList &list );
    /** Takes the shares of a queried host. */
    void slotShares( const QString &workgroup, const QString &host, const QStringList &list );
    /** If the scanner is running, this slot is called. */
    void slotRunning( bool on );
    /** If an error occurred during network scan, this slot is called. */
    void slotError( const QString &message );
    /** Mounts a share. */
    void slotMountShare();
    /** Invokes the askpass dialog. */
    void slotAskPass();
    /** Invokes the preview dialog. */
    void slotPreview();

  private:
    /** The KConfig variable. */
    KConfig *m_config;
    /** Determines whether hidden shares are shown. */
    bool m_hidden;
    /** Determines whether IPC$ shares are shown. */
    bool m_ipc;
    /** Determines whether ADMIN$ shares are shown. */
    bool m_admin;
    /** Determines whether printer shares are shown. */
    bool m_printer;
    /** Determines whether the Type column is shown. */
    bool m_showType;
    /** Determines whether the Comment column is shown. */
    bool m_showComment;
    /** Holds the list of the workgroups and their masters. Is used by processGroups(). */
    QStringList m_groupData;
    /** The popup menu. */
    KPopupMenu *m_menu;
    /** Enumeration for the popup menu. */
    enum ID{ Title, Rescan, Abort, Search, Password, Preview, Mount };
    /** Holds whether the scanner is running or not. */
    bool m_scanner;
};


#endif
