/***************************************************************************
    smb4knetworktab.cpp  -  Displays information about a selected
    network item.
                             -------------------
    begin                : Sam Mr 15 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qapplication.h>

// KDE includes
#include <kiconloader.h>
#include <klocale.h>

// application specific includes
#include "smb4knetworktab.h"
#include "../core/smb4kscanner.h"
#include "../smb4k.h"

Smb4KNetworkTab::Smb4KNetworkTab( QWidget *parent, const char *name ) : QWidget( parent, name )
{
  QGridLayout *grid = new QGridLayout( this );
  grid->setMargin( 30 );

  QLabel *picLabel = new QLabel( this );
  picLabel->setPixmap( DesktopIcon( "network", 32 ) );
  picLabel->setFixedWidth( 64 );
  picLabel->setMargin( 5 );

  QLabel *topLabel = new QLabel( i18n( "<font size=+2><b>Network</b></font>" ), this );

  QFrame *line = new QFrame( this );
  line->setFrameShadow( QFrame::Sunken );
  line->setFrameShape( QFrame::HLine );

  QSpacerItem *spacer1 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );
  
  m_workgroupLabel = new QLabel( QString::null, this );
  m_workgroup = new KSqueezedTextLabel( QString::null, this );

  m_masterLabel = new QLabel( QString::null, this );
  m_master = new KSqueezedTextLabel( QString::null, this );

  QSpacerItem *spacer2 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );

  m_hostLabel = new QLabel( QString::null, this );
  m_host = new KSqueezedTextLabel( QString::null, this );

  m_osLabel = new QLabel( QString::null, this );
  m_os = new KSqueezedTextLabel( QString::null, this );

  m_serverLabel = new QLabel( QString::null, this );
  m_server = new KSqueezedTextLabel( QString::null, this );

  QSpacerItem *spacer3 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );

  m_shareLabel = new QLabel( QString::null, this );
  m_share = new KSqueezedTextLabel( QString::null, this );

  m_typeLabel = new QLabel( QString::null, this );
  m_type = new KSqueezedTextLabel( QString::null, this );

  QSpacerItem *spacer4 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Expanding );
  
  grid->addMultiCellWidget( picLabel, 0, 1, 0, 0, 0 );
  grid->addMultiCellWidget( topLabel, 0, 1, 1, 4, 0 );
  grid->addMultiCellWidget( line, 2, 2, 0, 4, 0 );
  grid->addMultiCell( spacer1, 3, 3, 0, 4, 0 );
  grid->addWidget( m_workgroupLabel, 4, 1, 0 );
  grid->addMultiCellWidget( m_workgroup, 4, 4, 2, 4, 0 );
  grid->addWidget( m_masterLabel, 5, 1, 0 );
  grid->addMultiCellWidget( m_master, 5, 5, 2, 4, 0 );
  grid->addMultiCell( spacer2, 6, 6, 0, 4, 0 );
  grid->addWidget( m_hostLabel, 7, 1, 0 );
  grid->addMultiCellWidget( m_host, 7, 7, 2, 4, 0 );
  grid->addWidget( m_osLabel, 8, 1, 0 );
  grid->addMultiCellWidget( m_os, 8, 8, 2, 4, 0 );
  grid->addWidget( m_serverLabel, 9, 1, 0 );
  grid->addMultiCellWidget( m_server, 9, 9, 2, 4, 0 );
  grid->addMultiCell( spacer3, 10, 10, 0, 4, 0 );
  grid->addWidget( m_shareLabel, 11, 1, 0 );
  grid->addMultiCellWidget( m_share, 11, 11, 2, 4, 0 );
  grid->addWidget( m_typeLabel, 12, 1, 0 );
  grid->addMultiCellWidget( m_type, 12, 12, 2, 4, 0 );  
  grid->addMultiCell( spacer4, 13, 13, 0, 4, 0 );
  
  connect( sapp->scanner(), SIGNAL( workgroups( const QStringList & ) ), this, SLOT( slotWorkgroupList( const QStringList & ) ) );
  connect( sapp->scanner(), SIGNAL( killed() ), this, SLOT( slotKilled() ) );
  connect( sapp->scanner(), SIGNAL( info( const QString &, const QString & ) ), this, SLOT( slotInfo( const QString &, const QString & ) ) );  
}


Smb4KNetworkTab::~Smb4KNetworkTab()
{
}


void Smb4KNetworkTab::clear( int depth )
{
  if ( depth == 0 )
  {
    m_workgroup->setText( QString::null );
    m_workgroupLabel->setText( QString::null );
    m_master->setText( QString::null );
    m_masterLabel->setText( QString::null );
    m_host->setText( QString::null );
    m_hostLabel->setText( QString::null );
    m_os->setText( QString::null );
    m_osLabel->setText( QString::null);
    m_server->setText( QString::null );
    m_serverLabel->setText( QString::null );
    m_share->setText( QString::null );
    m_shareLabel->setText( QString::null );
    m_type->setText( QString::null );
    m_typeLabel->setText( QString::null );
  }
  if ( depth == 1 )
  {
    m_host->setText( QString::null );
    m_share->setText( QString::null );
    m_shareLabel->setText( QString::null );
    m_type->setText( QString::null );
    m_typeLabel->setText( QString::null );
  }
  if ( depth == 2 )
  {
    m_share->setText( QString::null );
    m_type->setText( QString::null );
  }
}

/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////


void Smb4KNetworkTab::slotItemInfo( QListViewItem *item )
{
  if ( item )
  {
    // Copy the old item. We need it to later on to check  whether the
    // wantMoreInfo() signal has to be emitted or not.
    QListViewItem *oldItem = m_current;
        
    // Now build up the display.
    m_current = item;
    clear( m_current->depth() );
    if ( m_current->depth() == 0 )
    {
      m_workgroupLabel->setText( i18n( "Workgroup:" ) );
      m_masterLabel->setText( i18n( "Master:" ) );
      m_workgroup->setText( m_current->text( 0 ) );
      if ( m_data.contains( m_current->text( 0 ) ) != 0 )
      {
        QStringList::ConstIterator it = m_data.find( m_current->text( 0 ) );
        it--;
        m_master->setText( *it );
      }
      else
        m_master->setText( i18n( "Unknown" ) );
    }
    else if ( m_current->depth() == 1 )
    {
      bool search = false;
      if ( oldItem->depth() == 0 )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->text( 0 ), m_current->text( 0 ) );
      }
      else if ( ( oldItem->depth() == 1 && oldItem->text( 0 ) != m_current->text( 0 ) ) || m_hostLabel->text().isEmpty() )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->text( 0 ), m_current->text( 0 ) );
      }
      else if ( ( oldItem->depth() == 2  && oldItem->parent()->text( 0 ) != m_current->text( 0 ) ) || m_hostLabel->text().isEmpty() )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->text( 0 ), m_current->text( 0 ) );
      }

      m_workgroupLabel->setText( i18n( "Workgroup:" ) );
      m_masterLabel->setText( i18n( "Master:" ) );
      QString host( m_current->text( 0 ) );
      if ( !m_current->text( 2 ).isEmpty() )
        host.append( "   (" ).append( m_current->text( 2 ) ).append( ")" );
      m_workgroup->setText( m_current->parent()->text( 0 ) );
      if ( m_data.contains( m_current->parent()->text( 0 ) ) != 0 )
      {
        QStringList::ConstIterator it = m_data.find( m_current->parent()->text( 0 ) );
        it--;
        m_master->setText( *it );
      }
      else
        m_master->setText( i18n( "Unknown" ) );
      m_hostLabel->setText( i18n( "Host:" ) );
      m_osLabel->setText( i18n( "OS:" ) );
      m_serverLabel->setText( i18n( "Server:" ) );
      m_host->setText( host );
      if ( search )
      {
        m_os->setText( i18n( "Searching..." ) );
        m_server->setText( i18n( "Searching..." ) );
      }
    }
    else if ( m_current->depth() == 2 )
    {
      bool search = false;
      if ( oldItem->depth() == 0 )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->parent()->text( 0 ), m_current->parent()->text( 0 ) );
      }
      else if ( ( oldItem->depth() == 1 && oldItem->text( 0 ) != m_current->parent()->text( 0 ) ) || m_hostLabel->text().isEmpty() )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->parent()->text( 0 ), m_current->parent()->text( 0 ) );
      }
      else if ( ( oldItem->depth() == 2 && oldItem->parent()->text( 0 ) != m_current->parent()->text( 0 ) ) || m_hostLabel->text().isEmpty() )
      {
        search = true;
        sapp->scanner()->getInfo( m_current->parent()->parent()->text( 0 ), m_current->parent()->text( 0 ) );
      }

      m_workgroupLabel->setText( i18n( "Workgroup:" ) );
      m_masterLabel->setText( i18n( "Master:" ) );
      m_hostLabel->setText( i18n( "Host:" ) );
      m_osLabel->setText( i18n( "OS:" ) );
      m_serverLabel->setText( i18n( "Server:" ) );
      QString host( m_current->parent()->text( 0 ) );
      if ( !m_current->parent()->text( 2 ).isEmpty() )
        host.append( "   (" ).append( m_current->parent()->text( 2 ) ).append( ")" );
      QString share( m_current->text( 0 ) );
      if ( !m_current->text( 2 ).isEmpty() )
        share.append( "   (" ).append( m_current->text( 2 ) ).append( ")" );
      m_workgroup->setText( m_current->parent()->parent()->text( 0 ) );
      if ( m_data.contains( m_current->parent()->parent()->text( 0 ) ) != 0 )
      {
        QStringList::ConstIterator it = m_data.find( m_current->parent()->parent()->text( 0 ) );
        it--;
        m_master->setText( *it );
      }
      else
        m_master->setText( i18n( "Unknown" ) );
      if ( m_host->text().contains( host ) == 0 )
      {
        m_host->setText( host );
        if ( search )
        {
          m_os->setText( i18n( "Searching..." ) );
          m_server->setText( i18n( "Searching..." ) );
        }
      }
      m_shareLabel->setText( i18n( "Share:" ) );
      m_share->setText( share );
      m_typeLabel->setText( i18n( "Type:" ) );
      m_type->setText( m_current->text( 1 ) );
    }
  }
  else
    clear( 0 );
}


void Smb4KNetworkTab::slotWorkgroupList( const QStringList &wglist )
{
  m_data = wglist;
}


void Smb4KNetworkTab::slotInfo( const QString &server, const QString &os )
{
  QString newOS;
  QString newServer;
  if ( os.contains( "Unix" ) != 0 )
    newOS = QString( "Linux/" ).append( os );
  else if ( os.isEmpty() )
    newOS = i18n( "Unknown" );
  else
    newOS = os;
  if ( server.isEmpty() )
    newServer = i18n( "Unknown" );
  else
    newServer = server;  

  // Now display everything:
  if ( m_current->depth() == 1 )
  {
    m_os->setText( newOS );
    m_server->setText( newServer );
  }
  else if ( m_current->depth() == 2 )
  {
    m_os->setText( newOS );
    m_server->setText( newServer );
  }  
}


void Smb4KNetworkTab::slotKilled()
{
  if ( m_current && m_current->depth() >= 1 )
  {
    m_os->setText( i18n( "Unknown" ) );
    m_server->setText( i18n( "Unknown" ) );
  }
}


void Smb4KNetworkTab::slotClear()
{
  clear( 0 );
}


#include "smb4knetworktab.moc"
