/***************************************************************************
    smb4ksharetab.cpp  -  description
                             -------------------
    begin                : Mit Mr 12 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qlayout.h>
#include <qdir.h>
#include <qregexp.h>

// KDE includes
#include <klocale.h>
#include <kiconloader.h>

// application specific includes
#include "smb4ksharetab.h"

Smb4KShareTab::Smb4KShareTab( QWidget *parent, const char *name ) : QWidget( parent, name )
{
  QGridLayout *grid = new QGridLayout( this );
  grid->setMargin( 30 );

  QLabel *picLabel = new QLabel( this );
  picLabel->setPixmap( DesktopIcon( "hdd_mount", 32 ) );
  picLabel->setFixedWidth( 64 );
  picLabel->setMargin( 5 );

  QLabel *topLabel = new QLabel( i18n( "<font size=+2><b>Share</b></font>" ), this );

  QFrame *line1 = new QFrame( this );
  line1->setFrameShadow( QFrame::Sunken );
  line1->setFrameShape( QFrame::HLine );

  QSpacerItem *spacer1 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );

  m_shareLabel = new QLabel( QString::null, this );
  m_share = new KSqueezedTextLabel( QString::null, this );

  m_mountpointLabel = new QLabel( QString::null, this );
  m_mountpoint = new KSqueezedTextLabel( QString::null, this );

  QSpacerItem *spacer2 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );

  m_totalLabel = new QLabel( QString::null, this );
  m_total = new KSqueezedTextLabel( QString::null, this );
  
  m_freeLabel = new QLabel( QString::null, this );
  m_free = new KSqueezedTextLabel( QString::null, this );
  
  m_usedLabel = new QLabel( QString::null, this );
  m_used = new KSqueezedTextLabel( QString::null, this );

  QSpacerItem *spacer3 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );

  m_usageLabel = new QLabel( QString::null, this );
  m_usage = new KProgress( 100, this );
  m_usage->setMaximumWidth( 100 );
  m_usage->setTextEnabled( true );

  QSpacerItem *spacer4 = new QSpacerItem( 10, 20, QSizePolicy::Preferred, QSizePolicy::Preferred );
  
  m_contentsLabel = new QLabel( QString::null, this );
  m_contents = new KSqueezedTextLabel( QString::null, this );
  
  QSpacerItem *spacer5 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Expanding );

  grid->addMultiCellWidget( picLabel, 0, 1, 0, 0, 0 );
  grid->addMultiCellWidget( topLabel, 0, 1, 1, 4, 0 );
  grid->addMultiCellWidget( line1, 2, 2, 0, 4, 0 );
  grid->addMultiCell( spacer1, 3, 3, 0, 4, 0 );
  grid->addWidget( m_shareLabel, 4, 1, 0 );
  grid->addMultiCellWidget( m_share, 4, 4, 2, 4, 0 );
  grid->addWidget( m_mountpointLabel, 5, 1, 0 );
  grid->addMultiCellWidget( m_mountpoint, 5, 5, 2, 4, 0 );
  grid->addMultiCell( spacer2, 6, 6, 0, 4, 0 );
  grid->addWidget( m_usageLabel, 7, 1, 0 );
  grid->addMultiCellWidget( m_usage, 7, 7, 2, 4, 0 );
  grid->addMultiCell( spacer3, 8, 8, 0, 4 );
  grid->addWidget( m_totalLabel, 9, 1, 0 );
  grid->addMultiCellWidget( m_total, 9, 9, 2, 4, 0 );
  grid->addWidget( m_freeLabel, 10, 1, 0 );
  grid->addMultiCellWidget( m_free, 10, 10, 2, 4, 0 );
  grid->addWidget( m_usedLabel, 11, 1, 0 );
  grid->addMultiCellWidget( m_used, 11, 11, 2, 4, 0 );
  grid->addMultiCell( spacer4, 12, 12, 0, 4, 0 );
  grid->addWidget( m_contentsLabel, 13, 1, 0 );
  grid->addMultiCellWidget( m_contents, 13, 13, 2, 4, 0 );
  grid->addMultiCell( spacer5, 14, 14, 0, 4, 0 );

  clear();

  m_proc = new KProcess();
  m_proc->setUseShell( true );
  m_command = QString::null;

  m_timer = new QTimer( this );
  hasItem = false;

  // Internal connections.
  connect( m_proc, SIGNAL( receivedStdout( KProcess *, char *, int ) ), SLOT( slotReceivedStdout( KProcess *, char *, int ) ) );
  connect( m_proc, SIGNAL( processExited( KProcess * ) ), SLOT( slotProcessExited( KProcess * ) ) );
  connect( m_timer, SIGNAL( timeout() ), SLOT( monitor() ) );

  m_timer->start( 1000, false );
}


Smb4KShareTab::~Smb4KShareTab()
{
  delete m_proc;
}


void Smb4KShareTab::init( const QString &item )
{
  m_current = item;
  hasItem = true;

  QString entry = item.section( ":", 1, 1 ).stripWhiteSpace();
  
  if ( entry.contains( " " ) == 1 )
    m_command.append( "df -P -k | grep " ).append( KProcess::quote( entry.replace( QRegExp( " " ), "_" ) ) );
  else
    m_command.append( "df -P -k | grep " ).append( KProcess::quote( entry ) );
  m_proc->operator<<( m_command );
  m_buffer = QString::null;
  m_proc->start( KProcess::NotifyOnExit, KProcess::Stdout );
}


void Smb4KShareTab::clear()
{
  m_shareLabel->setText( QString::null );
  m_share->setText( QString::null );
  m_mountpointLabel->setText( QString::null );
  m_mountpoint->setText( QString::null );
  m_totalLabel->setText( QString::null );
  m_total->setText( QString::null );
  m_freeLabel->setText( QString::null );
  m_free->setText( QString::null );
  m_usedLabel->setText( QString::null );
  m_used->setText( QString::null );
  m_usageLabel->setText( QString::null );
  m_usage->hide();
  m_contentsLabel->setText( QString::null );
  m_contents->setText( QString::null );

  hasItem = false;
}

/////////////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATIONS
/////////////////////////////////////////////////////////////////////////////


/****************************************************************************
   Internal connections.
****************************************************************************/


void Smb4KShareTab::monitor()
{
  if ( !m_current.isEmpty() && hasItem )
    init( m_current );
}


void Smb4KShareTab::slotReceivedStdout( KProcess *, char *buf, int len )
{
  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
}


void Smb4KShareTab::slotProcessExited( KProcess * )
{
  // Show all labels at once.
  m_shareLabel->setText( i18n( "Share:" ) );
  m_mountpointLabel->setText( i18n( "Mount Point:" ) );
  m_usageLabel->setText( i18n( "Usage:" ) );
  m_totalLabel->setText( i18n( "Total:" ) );
  m_freeLabel->setText( i18n( "Free:" ) );
  m_usedLabel->setText( i18n( "Used:" ) );
  m_contentsLabel->setText( i18n( "Contents:" ) );

  m_share->setText( m_current.section( ":", 0, 0 ).stripWhiteSpace() );
  m_mountpoint->setText( m_current.section( ":", 1, 1 ).stripWhiteSpace() );

  // Now look whether there occured an error. Usually the error
  // message starts with "df:". Let's look for this:
  if ( m_buffer.contains( "df:", true ) == 0 )
  {
    QStringList list = QStringList::split( ' ', m_buffer, false );
    QString tmp = list[4];
    tmp.truncate( tmp.length() - 1 );
    m_percentage = tmp.toInt();
    m_usage->setProgress( m_percentage );
    m_usage->show();
  
    double temp;
    QString dim;
    if ( list[1].toDouble() > 1023 )
    {
      temp = list[1].toDouble() / 1024;   // MB
      dim = " MB";
      if ( temp > 1023 )
      {
        temp = temp / 1024;               // GB
        dim = " GB";
      }
    }
    else
    {
      temp = list[1].toDouble();
      dim = " KB";
    }
    m_total->setText( QString( "%1" ).arg( temp, 0, 'f', 2 ). append( dim ).stripWhiteSpace() );
    if ( list[3].toDouble() > 1023 )
    {
      temp = list[3].toDouble() / 1024;   // MB
      dim = " MB";
      if ( temp > 1023 )
      {
        temp = temp / 1024;               // GB
        dim = " GB";
      }
    }
    else
    {
      temp = list[3].toDouble();
      dim = " KB";
    }
    m_free->setText( QString( "%1" ).arg( temp, 0, 'f', 2 ).append( dim ).stripWhiteSpace() );  
    if ( list[2].toDouble() > 1023 )
    {
      temp = list[2].toDouble() / 1024;   // MB
      dim = " MB";
      if ( temp > 1023 )
      {
        temp = temp / 1024;               // GB
        dim = " GB";
      }
    }
    else
    {
      temp = list[2].toDouble();
      dim = " KB";
    }
    m_used->setText( QString( "%1" ).arg( temp, 0, 'f', 2 ).append( dim ).stripWhiteSpace() );
  
    // Get the info about the directory.
    QDir *dir = new QDir( m_mountpoint->text() );
  
    // Substract 2 from all and dirs, because we do not want to count '.' and '..'.
    int all = 0;
    int dirs = 0;
    int files = 0;
    if ( dir->entryList( QDir::All, QDir::DefaultSort ).count() != 0 )
      all = dir->entryList( QDir::All, QDir::DefaultSort ).count() - 2;
    if ( dir->entryList( QDir::Dirs, QDir::DefaultSort ).count() != 0 )
      dirs = dir->entryList( QDir::Dirs, QDir::DefaultSort ).count() - 2;
    files = dir->entryList( QDir::Files, QDir::DefaultSort ).count();
  
    // Assemble the output.
    QString part1, part2, part3, output;
    if ( all == 1 )
      part1 = QString( i18n( "%1 Item   " ) ).arg( all );
    else
      part1 = QString( i18n( "%1 Items   " ) ).arg( all );
    if ( dirs == 1 )
      part2 = QString( i18n( "(%1 Directory, " ) ).arg( dirs );
    else
      part2 = QString( i18n( "(%1 Directories, " ) ).arg( dirs );
    if ( files == 1 )
      part3 = QString( i18n( "%1 File)" ) ).arg( files );
    else
      part3 = QString( i18n( "%1 Files)" ) ).arg( files );

    output.append( part1 ).append( part2 ).append( part3 );
    m_contents->setText( output );
  
    m_command = QString::null;
    m_proc->clearArguments();
  }
  else
  {
    // Here we have to put something that has to be done
    // when there was an IO error and the normal stuff isn't 
    // shown.
    // FIXME: Do this on version 0.4.0 or later.
  }
}


/****************************************************************************
   External connections.
****************************************************************************/


void Smb4KShareTab::slotInit( const QString &item )
{
  if ( !item.isEmpty() )
    init( item );
  else
    clear();
}


#include "smb4ksharetab.moc"
