/***************************************************************************
    smb4ksharewidget.h  -  The widget, that displays the mounted shares.
                             -------------------
    begin                : Sam Mr 1 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4KSHAREWIDGET_H
#define SMB4KSHAREWIDGET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qstringlist.h>
#include <qpainter.h>
#include <qpalette.h>

// KDE includes
#include <kiconview.h>
#include <kconfig.h>
#include <kprocess.h>
#include <kpopupmenu.h>
#include <kiconloader.h>

/** 
 * This class provides the ability display the mounted shares
 * and manipulate them.
 */
class Smb4KShareWidgetItem;
class Smb4KShareWidget : public KIconView
{
  Q_OBJECT

  friend class Smb4KApp;
  friend class Smb4KShareWidgetItem;
  
  public:
    /** The constructor. */
    Smb4KShareWidget( QWidget *parent = 0, const char *name = 0 );
    /** The destructor. */
    ~Smb4KShareWidget();
    /** Reads the options. */
    void readOptions();

  signals:
    /** Is emitted, when a share has been unmounted. */
    void itemUnmounted();
    /** Emits the info about the currently selected share */
    void info( const QString &item );
    
  protected slots:
    /** Is called by the menu. Does the unmounting. */
    void slotUnmountShare();
    /** Is called by the menu. Unmounts all shares at once. */
    void slotUnmountAllShares();
    /** Is called, if an error occured during mounting. */
    void slotReceivedError( const QString &output, int errorValue );
    /** Is called to import mounted shares into the share view. */
    void slotImported( const QStringList &data );
    /** Is called, when a share has been mounted. */
    void slotMounted( const QString &data );
    /** Is called, when a share has been unmounted. */
    void slotUnmounted( const QString &item );
    /** Is called, when all shares have been unmounted. */
    void slotUnmountedAll( const QStringList &shares );   
    /** Is called, when a item is executed. Starts the filemanager. */
    void slotItemExecuted( QIconViewItem *item );
    /** Is called, when a share is to be opened with the filemanager.
        This slot behaves exactly the same as slotItemExecuted(), except
        this is an external one. */
    void slotOpenFilemanager();
    /** Is called, every time a mouse button has been pressed. */
    void slotMouseButtonPressed( int button, QIconViewItem *item, const QPoint &pos );
    /** Is called, when the the selection in the icon view changed. */
    void slotSelectionChanged( QIconViewItem *item );
    
  private:
    /** This function is called by the destructor to do last-second
        stuff such as saving the list of mounted shares and to unmount
        all shares. */
    void exit();
    /** The KConfig variable. */
    KConfig *m_config;
    /** Holds the display style. */
    QString m_display;
    /** Holds whether the mount point should be shown as description. */
    bool m_mountpoint;
    /** The popup menu. */
    KPopupMenu *m_menu;
    enum ID{ Title, Unmount, UnmountAll, Konqueror, IconView, ListView };
    QString m_workgroup, m_host, m_share;
    /** Holds the uid of the user. */
    int m_uid;
    /** Holds the gid of the user. */
    int m_gid;
    /** The list of displayed (i.e. mounted) shares. */
    QStringList m_shares;
    /** Determines whether all shares that were mounted should
        be shown. */
    bool m_showExternal;
    /** If the options are read this value is true. */
    bool m_read;
};

class Smb4KShareWidget;
/** 
 * This class handles the icon view items that are to be displayed.
 */
class Smb4KShareWidgetItem : public KIconViewItem
{
  public:
    /** The constructor. It constructs a Smb4KShareWidgetItem with parent parent. */
    Smb4KShareWidgetItem( Smb4KShareWidget *parent = 0, const QString &text = QString::null, const QString &iconName = QString::null, const QString &alignment = QString::null, bool external = false );
    
    /** The destructor. */
    virtual ~Smb4KShareWidgetItem();
    
  protected:
    /** Reimplemented from QIconViewItem. */
    void paintItem( QPainter *p, const QColorGroup &cg );
  
  private:
    /** Determines whether the share that is going to be imported, belongs to the
        user or not. */
    bool m_externalMount;  
};
#endif
